/*
 *       ͻ
 *              Chi Programmer's Editor, Borland C++ 3.1       
 *                     
 *           Copyright (C) 1990,1991,1993                      
 *           Author: Fook H. Eng, 201 West Hilliard Avenue     
 *                   Monterey Park, CA 91754, 818 572-9220     
 *                                                             
 *       ͼ
 */
#include    <stdio.h>
#include    <string.h>
#include    <alloc.h>
#include    "vartypes.h"
#include    "memory.h"
#include    "utils.h"
#include    "keys.h"
#include    "blocks.h"
#include    "cache.h"
#include    "initial.h"
#include    "files.h"
#include    "screen.h"
#include    "windows.h"

static  void extractblock(void);
static  void insert_block(BlockType *);
static  void blockoff(void);
static  void blockbegin(void);
static  void blockend(void);
static  pLineDesc linecopy(pLineDesc);
static BlockType *  blockcopy(pLineDesc, pLineDesc);
static  void do_blockcopy(void);
static  void adjustlines(WinDesc *);
static  void lineshift(pLineDesc, int);
static  void blockshift(int);
static  void extractblock(void);
static  void insert_block(BlockType *);
static  void blockmove(void);
static  void editmacro(void);




int  in_block(pLineDesc p)
{
    pLineDesc  q = blockfro;

    if (!blockvalid()) return FALSE;

    while (q != blockto && q != p) q = q->pfwd;

    return (q == p);
}


int     in_window(WinDesc * pwin, pLineDesc pl)
{
    pLineDesc  p;

    if (pl == NULL) return FALSE;

    p = pwin->topline;
    while (p != NULL)
        if (p == pl) return TRUE;
        else p = p->paft;

    p = pwin->topline->pfwd;
    while (p != NULL)
        if (p == pl) return TRUE;
        else p = p->pfwd;

    return FALSE;
}


int     blockvalid()
{
    pLineDesc  p;
    int        i;

    if (blockfro == NULL || blockto == NULL) return FALSE;

    for (p = blockfro; p != NULL; p = p->pfwd) {
        for (i = 0; i < MAXCACHEBLK; i++)
            if (p == cache[i].qld)
                return FALSE;
        if (p == blockto) return TRUE;
    }

    return FALSE;
}

int     blockdefinedincurwin()
{
    if ( !blockvalid() )  return FALSE;
    else if ( !in_window(curwin, blockfro) ) return FALSE;
    else return TRUE;
}

static  void    blockoff()
{
    pLineDesc  pl;

    if (!blockvalid()) return;

    pl = blockfro;
    pl->inblock = FALSE;

    while (pl != blockto) {
        pl = pl->pfwd;
        pl->inblock = FALSE;
    }
}


static  void    blockbegin()
{
    blockoff();
    blockfro = curwin->curline;         /* repoint the blockfro pointer */
    if (!blockvalid()) blockto = NULL;
}

static  void    blockend()
{
    blockoff();
    blockto = curwin->curline;
    if (!blockvalid()) blockfro = NULL;
}

static pLineDesc linecopy(pLineDesc src)
{
    pLineDesc dest;

    if ((dest = maktxtdes(src->bufflen)) != NULL)
        memcpy(dest->ptxt, src->ptxt, src->bufflen);

    return dest;
}


static BlockType *  blockcopy(pLineDesc shead, pLineDesc stail)
{
    pLineDesc  sptr, dptr;
    BlockType  * pk;

    if ((pk = (BlockType *) malloc(sizeof(BlockType))) == NULL)
        return NULL;

    if ((pk->phead = linecopy(shead)) == NULL) return NULL;

    else {
        pk->ptail = pk->phead;
        pk->ptail->paft = NULL;
    }

    if ((sptr = shead) != stail) do {
        sptr = sptr->pfwd;
        if ((dptr = linecopy(sptr)) != NULL) {
            dptr->paft = pk->ptail;
            pk->ptail->pfwd = dptr;
            pk->ptail = dptr;
        }
        else {
            freeblock(pk->phead, pk->ptail);
            return NULL;
        }
    } while (sptr != stail);

    return pk;
}

static  void    do_blockcopy()
{
    BlockType *  pblock;

    if ( !blockvalid() )
        appcmdline("No block defined for block copy");

    else if ( in_block(curwin->curline) && curwin->curline != blockto )
        appcmdline("Can't copy copy block to within block");

    else if (( pblock = blockcopy(blockfro, blockto)) != NULL ) {
        insert_block(pblock);
        unmarkblock();
        realign();
    }

}


/*
 *  during block deletion, the window topline and/or the current may be
 *  within the marked block.  we must handle this delicately.
 *  used only by extract_block().
 */
static  void  adjustlines(WinDesc * pwin)
{
    if (in_block(pwin->topline)) {
        if (blockfro->paft != NULL)
            pwin->topline = blockfro->paft;
        else if (blockto->pfwd != NULL)
            pwin->topline = blockto->pfwd;
        else
            pwin->topline = maktxtdes(1);
    }

    if (in_block(pwin->curline)) {
        if (blockfro->paft != NULL)
            pwin->curline = blockfro->paft;
        else if (blockto->pfwd != NULL)
            pwin->curline = blockto->pfwd;
        else
            pwin->curline = pwin->topline;
    }
}

void    delete_block()
{
    if (!blockvalid()) {
        appcmdline("No blocks defined for deletion.");
        beep();
    }
    else {
        extractblock();
        if (pdelblk->phead != NULL)
            freeblock(pdelblk->phead, pdelblk->ptail);
        pdelblk->phead = blockfro;
        pdelblk->ptail = blockto;
        if (pdelblk != &delblklist[MAXDELBLOCK-1]) pdelblk++;
        else pdelblk = &delblklist[0];
        unmarkblock();
        realign();
        updcurflag = TRUE;
    }
}

static void lineshift(pLineDesc pl, int ch)
{
    int     i;

    if (ch == LEFT) {
        for (i = 0; i < pl->bufflen-1; i++)
            pl->ptxt[i] = pl->ptxt[i+1];
        pl->ptxt[pl->bufflen-1] = ' ';
    }
    else if (ch == RIGHT) {
        if (pl->ptxt[pl->bufflen-1] != ' ')
            if (!sizeline(pl, pl->bufflen+1)) {
                appcmdline("No more memory");
                return;
            }
        for (i = pl->bufflen-1; i > 0; i--)
            pl->ptxt[i] = pl->ptxt[i-1];
        pl->ptxt[0] = ' ';
    }
}


static void blockshift(int ch) {

    pLineDesc  p;

    p = blockfro;
    lineshift(p, ch);
    while (p != blockto) {
        p = p->pfwd;
        lineshift(p, ch);
    }
}

void    do_blockshift(int ch) {

    appcmdline("Press Esc to terminate block shift");
    do {
        blockshift(ch);
        refresh_screen();
        ch = getfcnkeychar();
    } while (ch == LEFT || ch == RIGHT);
    unmarkblock();
    appcmdline("");
}

static  void    extractblock()
{
    int     i;

     /* make sure curline and topline is tucked away before extraction */
    if ( in_window(curwin, blockfro) ) adjustlines(curwin);

    else if ( curwin != curwin->pfwd && in_window(curwin->pfwd, blockfro) )
        adjustlines(curwin->pfwd);

    else for (i = 0; i < 4; i++)
        if ( in_window(windxyz[i], blockfro) ) {
            adjustlines(windxyz[i]);
            break;
        }

    if (blockfro->paft != NULL) blockfro->paft->pfwd = blockto->pfwd;

    if (blockto->pfwd != NULL) blockto->pfwd->paft = blockfro->paft;
}

void    insert_delblock()
{
    if (pdelblk != &delblklist[0]) pdelblk--;
    else pdelblk = &delblklist[MAXDELBLOCK-1];

    if (pdelblk->phead != NULL) {
        pdelblk->ptail->pfwd = curwin->curline->pfwd;

        if (pdelblk->ptail->pfwd != NULL)
            pdelblk->ptail->pfwd->paft = pdelblk->ptail;

        pdelblk->phead->paft = curwin->curline;
        curwin->curline->pfwd = pdelblk->phead;

        pdelblk->phead = NULL;
        pdelblk->ptail = NULL;
    }
}

static  void insert_block(BlockType * pblk)
{
    pblk->ptail->pfwd = curwin->curline->pfwd;
    if (pblk->ptail->pfwd != NULL)
        pblk->ptail->pfwd->paft = pblk->ptail;

    pblk->phead->paft = curwin->curline;
    curwin->curline->pfwd = pblk->phead;
}


static  void    blockmove()
{
    BlockType   block;

    if (!blockvalid()) {
        appcmdline("block not defined");
        beep();
    }
    else if (in_block(curwin->curline)) {
        appcmdline("cannot move to within block");
        beep();
    }
    else {
        extractblock();
        block.phead = blockfro;
        block.ptail = blockto;
        insert_block( &block );
        unmarkblock();
        realign();
    }
    appcmdline("");
}


void    markblock()
{
    pLineDesc  pl;

    if (!blockvalid()) return;

    for (pl = blockfro; /* pl != NULL unnecessary */ ; pl = pl->pfwd) {
        pl->inblock = TRUE;
        if (pl == blockto) break;
    }
}


void    unmarkblock ()
{
    blockoff();
    blockfro = NULL;
    blockto  = NULL;
}


void    duplicate_line()
{
    pLineDesc  pdupln, cln = curwin->curline;

    if ((pdupln = maktxtdes(cln->bufflen)) != NULL) {
        memmove(pdupln->ptxt, cln->ptxt, cln->bufflen);
        pdupln->bufflen = cln->bufflen;
        pdupln->inblock = cln->inblock;
        pdupln->pfwd    = cln->pfwd;
        pdupln->paft    = cln;
        cln->pfwd->paft = pdupln;
        cln->pfwd       = pdupln;
    }
}


static  void    editmacro()
{
    pchar pstr;
    int   ch;

    sprintf(cmdline2, "Set or modify macro key F5 thru F12: ");
    appcmdline(cmdline2);
    gotoXY(strlen(cmdline2)+1, 1);

    if ((ch = getfcnkeychar() & 0xff) == NOTAVAILABLE) return;

    else {
        appcmdline("");

        switch (ch) {

            case F5:    case F6:    case F7:
            case F8:    case F9:    case F10:
                    sprintf(cmdline2, "F%d is currently defined as: ", ch-F5+5);
                    askmacro(cmdline2, macstr[ch-F5]);
                    break;

            case F11:   case F12:
                    sprintf(cmdline2, "F%d is currently defined as: ", ch-F11+11);
                    askmacro(cmdline2, macstr[ch-F11+6]);
                    break;

            default:
                    sprintf(cmdline2, "function key is %x", ch);
                    appcmdline(cmdline2);
                    beep();     break;
        }
    }
}


void    edit_k()
{
    int     ch, i = 0;
    pchar   cmdstr[] = {
" Beginblk  endblK  Copyblk  moVeblk  Pcacheblk  Hideblock  Ydelblk  SPmore  ESC ",
" editmAcro  Readfile  rEnamefile  Savefile  Writefile  eXiteditor   SPmore  ESC "
};
    do {
        write_commands(cmdstr[i]);
        while (!keypressed())   ;
        if ((ch = getctrlchar()) == NULL) {         /* if fnckey, get the */
            getinput();                             /* other and discard  */
            ch = NOTAVAILABLE;
        }
        i = 1 - i;
    } while (ch == ' ');

    appcmdline("");

    switch (ch) {
        case ctrla: editmacro();    break;

        case ctrlb: blockbegin();   break;

        case ctrlc: do_blockcopy(); break;

        case ctrlv: blockmove();    break;

        case ctrle: renamefile();   break;

        case ctrlh: unmarkblock();  break;

        case ctrlk: blockend();     break;

        case ctrlp: cache_text();   break;

        case ctrlr: readfile();     break;

        case ctrls: cpfilesave();   break;

        case ctrlw: cpfilewrite();  break;

        case ctrlx: cpfileexit();   break;

        case ctrly: delete_block(); break;

        default: break;
    }
    updcurflag = TRUE;
    realign();
    refresh_screen();
}