 '***************************************************************************
 '*                      E L A T O R   S Y S T E M S                        *
 '***************************************************************************
 '*                                                                         *
 '*  Module  . . :  FLDLST.MOD                                              *
 '*                                                                         *
 '*  Function  . :  Modules for field & list handling.                      *
 '*                                                                         *
 '*  Application :  General                                                 *
 '*                                                                         *
 '*  Author  . . :  Mike Finister                     Date . . :  20/12/91  *
 '*                                                                         *
 '***************************************************************************

 '***************************************************************************
 '* Include all definitions required.                                       *
 '***************************************************************************

  '$INCLUDE: 'fldlst.hdr'
  '$INCLUDE: 'stdio.hdr'
 
  '$INCLUDE: 'fldlst.def'
  '$INCLUDE: 'stdio.def'

 '**********
 '* DspFld *----------------------------------------------------------------*
 '**********                                                                |
 '|                                                                         |
 '|  Description:  Displays a given field on the screen.  Any trailing or   |
 '|                leading spaces, for alpha and numeric field respectively,|
 '|                are set to a fill character.  If the field is a numeric  |
 '|                with decimals then the decimal point is also added.  If  |
 '|                the field will fit on the given line, then a number of   |
 '|                #'s are displayed as a warning.                          |
 '|                                                                         |
 '|  Parameters :  fld      - Attributes of given field                     |
 '|                value$   - Current value of given field                  |
 '|                scrWid   - Width of display                              |
 '|                fColr    - Foreground color for field                    |
 '|                bColr    - Background color for field                    |
 '|                                                                         |
 '*-------------------------------------------------------------------------*
 '
SUB DspFld (fld AS FldType, value$, scrWid, fColr, bColr)

 '*-------------------------------------------------------------------------*
 '| Work out the field length, taking into account that if the field is     |
 '| numeric with decimals it's length is one more.                          |
 '*-------------------------------------------------------------------------*

  tmpLen = fld.length

  IF NOT (fld.char) AND fld.decs <> 0 THEN tmpLen = tmpLen + 1

 '*-------------------------------------------------------------------------*
 '| If the field cannot fit on the line show a string of #'s.               |
 '*-------------------------------------------------------------------------*

  IF (fld.col + tmpLen) - 1 > scrWid THEN
    IF fld.col > scrWid THEN fld.col = (scrWid - tmpLen) + 1
    tmpVal$ = STRING$((scrWid - fld.col) + 1, "#")
    BEEP
  ELSE
 
 '*-------------------------------------------------------------------------*
 '| Fill in any leading or trailing spaces on numeric & alpha fields        |
 '| respectively.                                                           |
 '*-------------------------------------------------------------------------*

    fillVal$ = value$

    IF fld.char THEN
      fillStr = tmpLen
      fillEnd = 1
      fillStep = -1
    ELSE
      fillStr = 1
      fillEnd = tmpLen
      fillStep = 1
    END IF

    FOR r = fillStr TO fillEnd STEP fillStep

      IF MID$(fillVal$, r, 1) = " " THEN
        MID$(fillVal$, r, 1) = fld.fillChar
      END IF

    NEXT r

 '*-------------------------------------------------------------------------*
 '| Put the decimal point into the output if the field is numeric & needs   |
 '| one.                                                                    |
 '*-------------------------------------------------------------------------*

    IF NOT (fld.char) AND fld.decs <> 0 THEN
      tmpVal$ = LEFT$(fillVal$, (fld.length - fld.decs))
      tmpVal$ = tmpVal$ + "." + RIGHT$(fillVal$, fld.decs)
    ELSE
      tmpVal$ = fillVal$
    END IF

  END IF
                                    
  DspTxt fld.row, fld.col, tmpVal$, fColr, bColr

END SUB

 '**********
 '* DspLst *----------------------------------------------------------------*
 '**********                                                                |
 '|                                                                         |
 '|  Description:  Sets up a the elements of alist menu on screen.          |
 '|                                                                         |
 '|  Parameters :  lst  - All information for given list                    |
 '|                opt$ - Array holding the list members to use             |
 '|                                                                         |
 '*-------------------------------------------------------------------------*
 '
SUB DspLst (lst AS ListType, opt$())

 '*-------------------------------------------------------------------------*
 '| Do a small amount of validation of list control data.                   |
 '*-------------------------------------------------------------------------*

  IF lst.start < 1 THEN lst.start = 1

  IF lst.finish < lst.start THEN lst.finish = lst.start

  IF lst.winStart < lst.start THEN lst.winStart = lst.start

  IF lst.winRow < 1 THEN lst.winRow = 1

  IF lst.winCol < 1 THEN lst.winCol = 1

  IF lst.ptr < lst.winStart THEN lst.ptr = lst.winStart

  winEnd = (lst.winStart - 1) + (lst.winRow * lst.winCol)
  IF lst.ptr > winEnd THEN lst.ptr = winEnd

 '*-------------------------------------------------------------------------*
 '| Actually display the list members.                                      |
 '*-------------------------------------------------------------------------*

  FOR c = 1 TO lst.winCol

    FOR r = 1 TO lst.winRow

      mbrPtr = (lst.winStart + (r - 1)) + ((c - 1) * lst.winRow)

      mbrRow = lst.row + ((r - 1) * lst.rowGap)
      mbrCol = lst.col + ((c - 1) * lst.colGap)
  
      IF mbrPtr > lst.finish THEN
        temp$ = STRING$(LEN(opt$(lst.start)), 32)
        DspTxt mbrRow, mbrCol, temp$, lst.fColr, lst.bColr
      ELSE
        DspTxt mbrRow, mbrCol, opt$(mbrPtr), lst.fColr, lst.bColr
      END IF

    NEXT r

  NEXT c

 '*-------------------------------------------------------------------------*
 '| Show whether more list members are available.                           |
 '*-------------------------------------------------------------------------*

  IF lst.showIcon THEN

    IF lst.winStart > lst.start THEN
  
      IF lst.winRow > 1 THEN
        temp$ = CHR$(24)
      ELSE
        temp$ = CHR$(27)
      END IF

    ELSE
      temp$ = " "
    END IF

    IF winEnd < lst.finish THEN

      IF lst.winRow > 1 THEN
        temp$ = temp$ + CHR$(25)
      ELSE
        temp$ = temp$ + CHR$(26)
      END IF

    END IF

    DspTxt lst.iconRow, lst.iconCol, "  ", lst.fColr, lst.bColr
    DspTxt lst.iconRow, lst.iconCol, temp$, lst.fColr, lst.bColr
  END IF

END SUB

 '**********
 '* FldVal *----------------------------------------------------------------*
 '**********                                                                |
 '|                                                                         |
 '|  Description:  Calculates, and returns, the actual value of a numeric   |
 '|                field.                                                   |
 '|                                                                         |
 '|  Parameters :  fld    - Attributes for given field                      |
 '|                value$ - Current value of given field                    |
 '|                                                                         |
 '|  Results    :  Sends back a double precision representation of the given|
 '|                numeric field.                                           |
 '|                                                                         |
 '*-------------------------------------------------------------------------*
 '
FUNCTION FldVal# (fld AS FldType, value$)

  FldVal# = (VAL(value$)) / (10 ^ fld.decs)

END FUNCTION

 '**********
 '* InzFld *----------------------------------------------------------------*
 '**********                                                                |
 '|                                                                         |
 '|  Description:  Initialises a field with blanks.                         |
 '|                                                                         |
 '|  Parameters :  fld    - Attributes of given field                       |
 '|                value$ - Current value of given field                    |
 '|                                                                         |
 '*-------------------------------------------------------------------------*
 '
SUB InzFld (fld AS FldType, value$)

  IF RTRIM$(value$) = "" THEN value$ = STRING$(fld.length, 32)

END SUB

 '**************
 '* PosFldCsr  *------------------------------------------------------------*
 '**************                                                            |
 '|                                                                         |
 '|  Description:  Positions the cursor under the correct character of a    |
 '|                user field.                                              |
 '|                                                                         |
 '|  Parameters :  fld    - Attributes of given field                       |
 '|                insFlg - Insert toggle flag                              |
 '|                                                                         |
 '|  Results    :  Sends back the actual screen column of the cursor.       |
 '|                                                                         |
 '*-------------------------------------------------------------------------*
 '
FUNCTION PosFldCsr (fld AS FldType, insFlg AS INTEGER)

 '*-------------------------------------------------------------------------*
 '| Place the cursor under the correct character. If the field is numeric   |
 '| with decimals then the position must be corrected if it has passed the  |
 '| decimal point.                                                          |
 '*-------------------------------------------------------------------------*

  csrCol = (fld.col + fld.ptr) - 1

  IF NOT (fld.char) AND fld.decs <> 0 THEN
    IF fld.ptr > (fld.length - fld.decs) THEN csrCol = csrCol + 1
  END IF

  IF insFlg THEN
    LOCATE fld.row, csrCol, 1, 4, 8
  ELSE
    LOCATE fld.row, csrCol, 1, 7, 8
  END IF

  PosFldCsr = csrColr

END FUNCTION

 '**********
 '* WrkFld *----------------------------------------------------------------*
 '**********                                                                |
 '|                                                                         |
 '|  Description:  Allows the user to "work with" the field.  All editing   |
 '|                that takes place is governed by both input mode as well  |
 '|                as field type.                                           |
 '|                                                                         |
 '|  Parameters :  fld      - Attributes of given field                     |
 '|                value$   - Current value of given field                  |
 '|                scrWid   - Width of display                              |
 '|                fColr    - Foreground color for field                    |
 '|                bColr    - Background color for field                    |
 '|                                                                         |
 '*-------------------------------------------------------------------------*
 '
SUB WrkFld (fld AS FldType, value$, scrWid, fColr, bColr)

 '*-------------------------------------------------------------------------*
 '| Set the insert flag to off.                                             |
 '*-------------------------------------------------------------------------*

  insFlg = FALSE

 '*-------------------------------------------------------------------------*
 '| Set up the correct 'virtual end' position for insert scroll.            |
 '*-------------------------------------------------------------------------*

  IF fld.char THEN
    virtEnd = fld.length
  ELSE
    virtEnd = 1
  END IF

 '*-------------------------------------------------------------------------*
 '| If the pointer is zero then set it correctly.                           |
 '*-------------------------------------------------------------------------*

  IF fld.ptr = 0 THEN fld.ptr = (fld.length + 1) - virtEnd

 '*-------------------------------------------------------------------------*
 '| Display the field & enter an endless loop.  Certain keystrokes will     |
 '| cause sub-program exit.                                                 |
 '*-------------------------------------------------------------------------*

  DspFld fld, value$, scrWid, fColr, bColr

  DO
 
 '*-------------------------------------------------------------------------*
 '| Place the cursor under the correct character. If the field is numeric   |
 '| with decimals then the position must be corrected if it has passed the  |
 '| decimal point.                                                          |
 '*-------------------------------------------------------------------------*
 
    csrCol = PosFldCsr(fld, insFlg)

 '*-------------------------------------------------------------------------*
 '| Wait for a keystroke.                                                   |
 '*-------------------------------------------------------------------------*
 
    DO

 '*-------------------------------------------------------------------------*
 '| If the cursor has moved position then reset it back.  This could happen |
 '| if event trapping is used & screen output takes place.                  |
 '*-------------------------------------------------------------------------*
    
      IF csrCol <> POS(0) OR fld.row <> CSRLIN THEN
        csrCol = PosFldCsr(fld, insFlg)
      END IF

      a$ = INKEY$
    LOOP UNTIL a$ <> ""

 '*-------------------------------------------------------------------------*
 '| Get the scan code of the keystroke & add 300 to it if extended ASCII    |
 '| code.                                                                   |
 '*-------------------------------------------------------------------------*
 
    a = ASC(RIGHT$(a$, 1))
    IF LEN(a$) = 2 THEN a = a + 300

 '*-------------------------------------------------------------------------*
 '| Act on given input.                                                     |
 '*-------------------------------------------------------------------------*
 
    SELECT CASE a

 '*-------------------------------------------------------------------------*
 '| If Tab, Enter or Escape have been pressed then set the action key field |
 '| and exit the sub-program.                                               |
 '*-------------------------------------------------------------------------*

      CASE FWDTAB, ENTER, ESCAPE

        fld.actKey = a
        EXIT SUB

 '*-------------------------------------------------------------------------*
 '| Process extended ASCII characters the same as above. Main program will  |
 '| have to take care of the 300 difference with proper code.               |
 '*-------------------------------------------------------------------------*
 
      CASE F1 TO F10, SHFTF1 TO ALTF10, F11 TO ALTF12

        fld.actKey = a
        EXIT SUB

 '*-------------------------------------------------------------------------*
 '| Many extended keys have no function here, but mat be used by the        |
 '| mainline code for some reason, so exit when these are pressed also.     |
 '*-------------------------------------------------------------------------*

      CASE CSRUP, CSRDN, PGUP, PGDN, CTRLHOME, CTRLEND, BACKTAB

        fld.actKey = a
        EXIT SUB

 '*-------------------------------------------------------------------------*
 '| Backspace will delete character to the left of cursor & scroll the field|
 '| depending on whether the field in alpha or not.                         |
 '*-------------------------------------------------------------------------*

      CASE BACKSPC

        IF fld.ptr = 1 THEN
          BEEP
        ELSE
       
          IF MID$(value$, fld.ptr - 1, 1) = " " AND NOT (fld.char) THEN
            BEEP
          ELSE

            tmpVal$ = ""

            IF fld.ptr > 2 THEN tmpVal$ = LEFT$(value$, fld.ptr - 2)
         
            tmpVal$ = tmpVal$ + RIGHT$(value$, (fld.length - fld.ptr) + 1)

            IF fld.char THEN
              tmpVal$ = tmpVal$ + " "
              fld.ptr = fld.ptr - 1
            ELSE
              tmpVal$ = " " + tmpVal$
            END IF

            value$ = tmpVal$
            DspFld fld, value$, scrWid, fColr, bColr
          END IF
     
        END IF

 '*-------------------------------------------------------------------------*
 '| Allow all characters for an alpha field. Must take care of correct input|
 '| mode, (ie. insert or overwrite).                                        |
 '*-------------------------------------------------------------------------*

      CASE 32 TO 126 AND fld.char

        IF NOT (insFlg) THEN
       
          IF fld.ptr = fld.length AND RIGHT$(value$, 1) <> " " THEN
            BEEP
          ELSE
            MID$(value$, fld.ptr, 1) = CHR$(a)
            IF fld.ptr <> fld.length THEN fld.ptr = fld.ptr + 1
          END IF

        ELSE
       
          IF MID$(value$, virtEnd, 1) <> " " THEN
            BEEP
          ELSE
            temp$ = value$
            value$ = LEFT$(temp$, fld.ptr - 1) + CHR$(a)
            value$ = value$ + MID$(temp$, fld.ptr, (fld.length - fld.ptr))
          END IF

        END IF
     
        DspFld fld, value$, scrWid, fColr, bColr

 '*-------------------------------------------------------------------------*
 '| Numeric fields are different so have their own CASE.                    |
 '*-------------------------------------------------------------------------*

      CASE 48 TO 57 AND NOT (fld.char)
      
          IF MID$(value$, virtEnd, 1) <> " " THEN
            BEEP
          ELSE

 '*- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -*
 '| If no numbers have been entered yet then simply placed the digit at the |
 '| end of the field and exit the case clause.                              |
 '*- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -*
         
            IF fld.ptr = fld.length AND RIGHT$(value$, 1) = " " THEN
              MID$(value$, fld.length, 1) = CHR$(a)
            ELSE
           
 '*- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -*
 '| If the cursor is not on or next to a number then signal that it is not  |
 '| allowed.                                                                |
 '*- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -*
           
              IF MID$(value$, fld.ptr, 2) = "  " THEN
                BEEP
              ELSE

 '*- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -*
 '| If the cursor is on a space and insert is on then signal that it is not |
 '| allowed.                                                                |
 '*- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -*

                IF MID$(value$, fld.ptr, 1) = " " AND insFlg THEN
                  BEEP
                ELSE

 '*- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -*
 '| Save the current field value & set the virtual pointer for field        |
 '| scrolling dependent on input mode.                                      |
 '*- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -*
               
                  tmpVal$ = value$
               
                  IF insFlg THEN
                    tmpPtr = fld.ptr - 1
                  ELSE
                    tmpPtr = fld.ptr
                  END IF

                  value$ = MID$(tmpVal$, 2, tmpPtr - 1) + CHR$(a)

 '*- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -*
 '| Only add trailing characters if the pointer is not at the end of the    |
 '| field or if in insert mode.                                             |
 '*- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -*
             
                  IF fld.ptr <> fld.length OR insFlg THEN
                    value$ = value$ + RIGHT$(tmpVal$, (fld.length - tmpPtr))
                  END IF

                END IF
              END IF
            END IF
          END IF

        DspFld fld, value$, scrWid, fColr, bColr

 '*-------------------------------------------------------------------------*
 '| If Home is pressed then set the pointer to be the start of the field.   |
 '| Process differently for numeric fields though.                          |
 '*-------------------------------------------------------------------------*

      CASE HOME

        fld.ptr = 1

 '*- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -*
 '| If the field is numeric then scan it from left to right for the first   |
 '| non-space character and place the cursor there, unless it is the decimal|
 '| point where the pointer is moved on one more place.                     |
 '*- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -*
     
        IF NOT (fld.char) AND fld.decs <> 0 THEN

          FOR r = 1 TO fld.length

            IF MID$(value$, r, 1) <> " " THEN
           
              IF MID$(value$, r, 1) = "." THEN
                fld.ptr = r + 1
              ELSE
                fld.ptr = r
              END IF
            
              EXIT FOR

            END IF
        
          NEXT r

        END IF

 '*-------------------------------------------------------------------------*
 '| Cursor left                                                             |
 '*-------------------------------------------------------------------------*

      CASE CSRLT

        IF fld.ptr <> 1 THEN
          fld.ptr = fld.ptr - 1
        END IF

 '*-------------------------------------------------------------------------*
 '| Cursor right will move the cursor one position to the right. It will    |
 '| skip over the decimal point if the field is numeric.                    |
 '*-------------------------------------------------------------------------*

      CASE CSRRT

        IF fld.ptr <> fld.length THEN
          fld.ptr = fld.ptr + 1
        END IF

 '*-------------------------------------------------------------------------*
 '| If End is pressed move the pointer to the end of a numeric field, but to|
 '| the last character of a string.                                         |
 '*-------------------------------------------------------------------------*

      CASE ENDKEY

        IF NOT (fld.char) THEN
          fld.ptr = fld.length
        ELSE
        
          FOR r = fld.length TO 1 STEP -1
          
            IF MID$(value$, r, 1) <> " " THEN
              fld.ptr = r
              EXIT FOR
            END IF
        
          NEXT r
      
        END IF

 '*-------------------------------------------------------------------------*
 '| Insert will simply toggle the flag.                                     |
 '*-------------------------------------------------------------------------*

      CASE INSERT

        insFlg = NOT (insFlg)
   
 '*-------------------------------------------------------------------------*
 '| Delete will erase the character under the cursor & cause the field to   |
 '| scroll dependent on whether it is alpha or not.                         |
 '*-------------------------------------------------------------------------*

      CASE DELETE
              
        IF fld.char THEN
          tmpVal$ = RIGHT$(value$, (fld.length - fld.ptr) + 1)
        ELSE
          tmpVal$ = LEFT$(value$, fld.ptr)
        END IF


        IF RTRIM$(tmpVal$) = "" THEN
          BEEP
        ELSE
          tmpVal$ = ""
       
          IF fld.ptr <> 1 THEN tmpVal$ = LEFT$(value$, fld.ptr - 1)

          IF fld.ptr <> fld.length THEN
            tmpVal$ = tmpVal$ + RIGHT$(value$, fld.length - fld.ptr)
          END IF

          IF fld.char THEN
            tmpVal$ = tmpVal$ + " "
          ELSE
            tmpVal$ = " " + tmpVal$
          END IF

          value$ = tmpVal$
          DspFld fld, value$, scrWid, fColr, bColr
        END IF

      CASE ELSE

    END SELECT

  LOOP

END SUB

 '**********
 '* WrkLst *----------------------------------------------------------------*
 '**********                                                                |
 '|                                                                         |
 '|  Description:  Works with a list. Allows fill scrolling of members &    |
 '|                only exits when Enter, Escape or a function key is       |
 '|                pressed.                                                 |
 '|                                                                         |
 '|  Parameters :  lst    - Holds all information about given list          |
 '|                opt$() - Array holding list members to work with         |
 '|                                                                         |
 '*-------------------------------------------------------------------------*
 '
SUB WrkLst (lst AS ListType, opt$())


  mbrLmt = UBOUND(opt$)

 '*-------------------------------------------------------------------------*
 '| Loop around indefinitely.  Certain keystrokes trigger an EXIT SUB.      |
 '*-------------------------------------------------------------------------*

  DO

 '*-------------------------------------------------------------------------*
 '| Calculate the row for the given member to highlight and also work out   |
 '| the last element on the page in case it has changed through scrolling.  |
 '| Highlight the given list member.                                        |
 '*-------------------------------------------------------------------------*

    temp = (lst.ptr - lst.winStart)

    newRow = (temp MOD lst.winRow) * lst.rowGap
    newRow = lst.row + newRow

    newCol = (temp \ lst.winRow) * lst.colGap
    newCol = lst.col + newCol

    DspTxt newRow, newCol, opt$(lst.ptr), lst.bfColr, lst.bbColr

    winEnd = (lst.winStart - 1) + (lst.winRow * lst.winCol)

    IF winEnd > mbrLmt THEN winEnd = mbrLmt

 '*-------------------------------------------------------------------------*
 '| Wait for a key press.                                                   |
 '*-------------------------------------------------------------------------*

    DO
      a$ = INKEY$
    LOOP UNTIL a$ <> ""

 '*-------------------------------------------------------------------------*
 '| Get the scan code of the user input. RIGHT$ takes care of extended ASCII|
 '| codes. For extended codes 300 is added to differentiate from normal     |
 '| ASCII characters.                                                       |
 '*-------------------------------------------------------------------------*

    a = ASC(RIGHT$(a$, 1))

    IF LEN(a$) = 2 THEN a = a + 300

 '*-------------------------------------------------------------------------*
 '| Act on the given keystroke.                                             |
 '*-------------------------------------------------------------------------*

    SELECT CASE a

 '*-------------------------------------------------------------------------*
 '| If Tab, Enter or Escape have been pressed then set the action key field |
 '| and exit the sub-program.                                               |
 '*-------------------------------------------------------------------------*

      CASE FWDTAB, BACKTAB, ENTER, ESCAPE

        lst.actKey = a
        EXIT SUB
 
 '*-------------------------------------------------------------------------*
 '| Process extended ASCII characters the same as above. Main program will  |
 '| have to take care of the 300 difference with proper code.               |
 '*-------------------------------------------------------------------------*
 
      CASE F1 TO F10, SHFTF1 TO ALTF10, F11 TO ALTF12

        lst.actKey = a
        EXIT SUB

 '*-------------------------------------------------------------------------*
 '| If Home is pressed then set the pointer to be the start of the current  |
 '| window.                                                                 |
 '*-------------------------------------------------------------------------*

      CASE HOME

        DspTxt newRow, newCol, opt$(lst.ptr), lst.fColr, lst.bColr
        lst.ptr = lst.winStart

 '*-------------------------------------------------------------------------*
 '| If End is pressed move the pointer to the end of the window.            |
 '*-------------------------------------------------------------------------*

      CASE ENDKEY

        DspTxt newRow, newCol, opt$(lst.ptr), lst.fColr, lst.bColr
        lst.ptr = winEnd

 '*-------------------------------------------------------------------------*
 '| For Ctrl-Home move the pointer to the very start of the list.           |
 '*-------------------------------------------------------------------------*

      CASE CTRLHOME

        lst.ptr = lst.start
        lst.winStart = lst.start
        DspLst lst, opt$()

 '*-------------------------------------------------------------------------*
 '| For Ctrl-End move the pointer to the very end of the list.              |
 '*-------------------------------------------------------------------------*

      CASE CTRLEND

        lst.ptr = lst.finish
        lst.winStart = (lst.ptr + 1) - (lst.winRow * lst.winCol)
        DspLst lst, opt$()

 '*-------------------------------------------------------------------------*
 '| For Page-Up                                                             |
 '*-------------------------------------------------------------------------*

      CASE PGUP

        IF lst.winStart <> lst.start THEN
          lst.ptr = lst.winStart - (lst.winRow * lst.winCol)
          IF lst.ptr < lst.start THEN lst.ptr = lst.start
          lst.winStart = lst.ptr
          DspLst lst, opt$()
        END IF

 '*-------------------------------------------------------------------------*
 '| For Page-Down                                                           |
 '*-------------------------------------------------------------------------*

      CASE PGDN

        IF winEnd <> lst.finish THEN

          temp = winEnd + (lst.winRow * lst.winCol)
  
          IF temp > lst.finish THEN
            lst.ptr = (lst.finish + 1) - (lst.winRow * lst.winCol)
          ELSE
            lst.ptr = winEnd + 1
          END IF

          lst.winStart = lst.ptr
          DspLst lst, opt$()
        END IF

 '*-------------------------------------------------------------------------*
 '| For Cursor Up move the pointer on place up the list. If the first list  |
 '| element has already been reached move no further. If the window start   |
 '| has been passed then scroll the list down one place.                    |
 '*-------------------------------------------------------------------------*

      CASE CSRUP AND lst.winRow <> 1

        IF lst.ptr <> lst.start THEN
          DspTxt newRow, newCol, opt$(lst.ptr), lst.fColr, lst.bColr
          lst.ptr = lst.ptr - 1
  
          IF lst.ptr < lst.start THEN
            lst.ptr = lst.start
          ELSE
    
            IF lst.ptr < lst.winStart THEN
              lst.winStart = lst.ptr
              DspLst lst, opt$()
            END IF
  
          END IF
        END IF

 '*-------------------------------------------------------------------------*
 '| For Cursor Down move the pointer on place down the list.  If the end of |
 '| the list has been reached then leave the pointer as it is.  If the      |
 '| window end has been passed then scroll the list up by one place.        |
 '*-------------------------------------------------------------------------*

      CASE CSRDN AND lst.winRow <> 1

        IF lst.ptr <> lst.finish THEN
          DspTxt newRow, newCol, opt$(lst.ptr), lst.fColr, lst.bColr
          lst.ptr = lst.ptr + 1
   
          IF lst.ptr > lst.finish THEN
            lst.ptr = lst.finish
          ELSE
     
            IF lst.ptr > winEnd THEN
              lst.winStart = (lst.ptr + 1) - (lst.winRow * lst.winCol)
              DspLst lst, opt$()
            END IF

          END IF
        END IF

 '*-------------------------------------------------------------------------*
 '| For Cursor Left move the pointer one row to the left.  If this is off   |
 '| the screen then scroll the list one column to the right.                |
 '*-------------------------------------------------------------------------*

      CASE CSRLT AND lst.winCol <> 1
     
        DspTxt newRow, newCol, opt$(lst.ptr), lst.fColr, lst.bColr
             
        temp = lst.ptr - lst.winRow

        IF temp < win.start THEN
          temp = win.start
        END IF

        lst.ptr = temp
              
        IF temp < lst.winStart THEN
          lst.winStart = lst.winStart - lst.winRow
                
          IF lst.winStart < lst.start THEN
            lst.winStart = lst.start
            lst.ptr = lst.start
          END IF

        END IF
              
        DspLst lst, opt$()

 '*-------------------------------------------------------------------------*
 '| For Cursor Right move the pointer one row to the right.  If this is off |
 '| the screen then scroll the list one column to the left.                 |
 '*-------------------------------------------------------------------------*

      CASE CSRRT AND lst.winCol <> 1
    
        DspTxt newRow, newCol, opt$(lst.ptr), lst.fColr, lst.bColr
            
        temp = lst.ptr + lst.winRow

        IF temp > lst.finish THEN
          temp = lst.finish
        END IF

        lst.ptr = temp
             
        IF temp > winEnd THEN
          lst.winStart = lst.winStart + lst.winRow
        END IF
             
        DspLst lst, opt$()

      CASE ELSE

    END SELECT

  LOOP

END SUB

