/*
 *       ͻ
 *              Chi Programmer's Editor, Borland C++ 3.1       
 *                     
 *           Copyright (C) 1990,1991,1993                      
 *           Author: Fook H. Eng, 201 West Hilliard Avenue     
 *                   Monterey Park, CA 91754, 818 572-9220     
 *                                                             
 *       ͼ
 */
#include    <stdio.h>
#include    <alloc.h>
#include    <stdlib.h>
#include    <dos.h>
#include    <string.h>

#include    "vartypes.h"
#include    "memory.h"
#include    "utils.h"
#include    "screen.h"
#include    "files.h"

static REGS regs;

static void  read_macros(void);
static int  enhanced_kbd_supported(void);
static void set_crtmode_parameters(void);
static void set_virtual_screen(void);

static void  read_macros() {

    if (file_exist("chi.mac")) load_macros("chi.mac");
    else {
        strcpy(macstr[0], "\x11\x19");          /* ^Q^Y = delete stream */
        strcpy(macstr[1], "\x0b\x12");          /* ^K^R = read a file   */
        strcpy(macstr[2], "\x0b\x02");          /* ^K^B = block begin   */
        strcpy(macstr[3], "\x0b\x0b");          /* ^K^K = block end     */
        strcpy(macstr[4], "\x0b\x13");          /* ^K^S = file save     */
        strcpy(macstr[5], "\x0b\x18");          /* ^K^X = program exit  */
        strcpy(macstr[6], "\x11\x06");          /* ^Q^F = find string   */
        strcpy(macstr[7], "\x11\x01");          /* ^Q^A = find-replace  */
    }
}

static int  enhanced_kbd_supported() {

    regs.x.cx = 0xe00d;             // scan-char code = keypad ENTER
    regs.x.ax = 0x0500;
    int86(KEYBD, &regs, &regs);     // keyboard write

    regs.x.ax = 0x1000;
    int86(KEYBD, &regs, &regs);     // read it back

    return (regs.x.ax == 0xe00d ? TRUE : FALSE);
}

static void set_crtmode_parameters() {

    regs.x.ax = 0x0f00;
    int86 (VIDEO, &regs, &regs);
    colormode = regs.h.al != 0x07;
    defnocols = regs.h.ah;

    if (colormode) {
        regs.x.ax = 0x1130;
        int86 (VIDEO, &regs, &regs);
        defnorows = min( max (regs.h.dl, 24), 74) + 1;
        regs.x.ax = 0x0800;
        int86 (VIDEO, &regs, &regs);
        txtcolor = regs.h.ah;           /* retain attribute as found */
        bordcolor = (0xf0 & regs.h.ah) >> 4 | (0x07 & regs.h.ah) << 4;
        blockcolor = bordcolor;
        bright = bordcolor;
    }

    else {
        defnorows = 25;
        txtcolor = 0x0b;
        bordcolor = 0x70;
        blockcolor = 0x30;
        bright = bordcolor + 1;
    }

    blankrow = (int *) malloc (defnocols * sizeof(int));
    for (int i = 0; i < defnocols; i++)
        blankrow[i] = txtcolor << 8;

}

static void set_virtual_screen() {

    register  int  i, j;

    vscreen = (int **) malloc (defnorows * sizeof(int *));
    for (i = 0; i < defnorows; i++)
        vscreen[i] = (int * ) malloc (defnocols * sizeof(int));
    for (i = 0; i < defnorows; i++)
        for (j = 0; j < defnocols; j++)
            vscreen[i][j] = txtcolor << 8;
}

void initialize() {

    int     i, j;
    WinDesc * p;

    abortcmd = FALSE;
    blockfro = blockto = NULL;
    circin = circout = 0;
    cmdcol = 1;
    replacestr[0] = '\0';
    searchstr[0]  = '\0';
    rundown = FALSE;
    updcurflag = TRUE;
    tabsize = 4;

    read_macros();

    for (i = 0; i < MAXCACHEBLK; i++) {
        cache[i].phd = NULL;
        cache[i].ptl = NULL;
        if ((cache[i].qld = maktxtdes(80)) != NULL)
            cache[i].qld->inblock = TRUE;
    }

    ppdelln = &pdellnlist[0];
    for (i = 0; i < MAXDELLINE; i++) pdellnlist[i] = NULL;

    pdelblk = &delblklist[0];
    for (i = 0; i < MAXDELBLOCK; i++) {
        delblklist[i].phead = NULL;
        delblklist[i].ptail = NULL;
    }

    if (enhanced_kbd_supported()) {
        getkey = 0x1000;
        chkkey = 0x1100;
    }
    else {
        getkey = 0x0000;
        chkkey = 0x0100;
    }

    set_crtmode_parameters();

    set_virtual_screen();

    physcrrows = defnorows;
    physcrcols = defnocols;
    logscrrows = physcrrows - 1;
    logscrcols = defnocols;
    logtopscr  = 2;             /* windows have a fname & comm line */
    cmdcol     = 1;

    /* allocate window structures */
    winstack = NULL;
    for (i = 0; i < MAXWINSTACK; i++) {
        p = (WinDesc *) malloc(sizeof(WinDesc));
        p->pfwd = winstack;
        winstack = p;
    }

    for (i = 0; i < 4; i++)
        windxyz[i] = creat_window(logtopscr, logscrrows, "nofile", 1, 1);

    window1 = creat_window(logtopscr, logscrrows, "nofile", 1, 1);
    window1->pfwd = window1;

    curwin = window1;
    extra.ptr  = curwin->curline;
    extra.dlen = 0x10;
    memset(extra.txt, ' ', MAXLINELENGTH);

    if (mymouse.MouseReset() != 0) {
        mouse_Exist = TRUE;
        mymouse.MouseTextType(0x77ff, 0x7700);
        mymouse.MouseYLimit(0x10, (defnorows-1) * 8);   // exclude top 2 lines
        mymouse.MouseXLimit(0, (defnocols-1) * 8);
        mymouse.MouseSetPosn(0, 0x10);                  // upper left corner
        mymouse.MouseShow();
    }
    else
        mouse_Exist = FALSE;

    set_cursor_type(TRUE);
    updphyscr();
}