'***************************************************************************
'*                                                                         *
'* Name  . . . :  INPDATA                                                  *
'*                                                                         *
'* Version . . :  1.00                                                     *
'*                                                                         *
'* Compiler  . :  Microsoft QuickBASIC 4.50                                *
'*                                                                         *
'* Text  . . . :  Allows a user to input data into a file                  * 
'*                                                                         *
'* Options . . :  Compiled within environment using TOOLBOX library.       *
'*                                                                         *
'* Author  . . :  Mike Finister                     Date . . :  25/02/1993 *
'*                                                                         *
'***************************************************************************
'* (c) Copyright 1993 Elator Systems                                       *
'*     All rights reserved                                                 *
'***************************************************************************

'***************************************************************************
'*                           A M E N D M E N T S                           *
'***************************************************************************
'* Name :  xxx...                         Date :  xx/xx/xxxx  Ref :  xxxxx *
'*                                                                         *
'* Text :                                                                  *
'*                                                                         *
'*                                                                         *
'***************************************************************************

'***************************************************************************
'* Set all non-qualified fields to be integer.                             *
'***************************************************************************

  DEFINT A-Z

'***************************************************************************
'* Include all of the necessary header & definition files for the TOOLBOX  *
'* library.                                                                *
'***************************************************************************

  '$INCLUDE: 'stdio.hdr'
  '$INCLUDE: 'bios.hdr'
  '$INCLUDE: 'fldlst.hdr'

  '$INCLUDE: 'stdio.def'
  '$INCLUDE: 'bios.def'
  '$INCLUDE: 'fldlst.def'

'***************************************************************************
'* Define special data structures.                                         *
'***************************************************************************

  TYPE LabelType
    prdCode     AS STRING * 15
    ordNo       AS STRING * 8
    drumNo      AS STRING * 9
    metres      AS STRING * 4
    NTVFlag     AS STRING * 1
    prdDsc1     AS STRING * 30
    prdDsc2     AS STRING * 30
    weight      AS STRING * 5
    cableDsc    AS STRING * 10
    pairageDsc  AS STRING * 13
    date        AS STRING * 8
  END TYPE
 
'***************************************************************************
'* Declare application routines.                                           *
'***************************************************************************

  DECLARE SUB ExceptionHandler ()
  DECLARE SUB HelpText (helpName AS STRING)
  DECLARE SUB InitialiseProgram ()
  DECLARE SUB Main ()
  DECLARE SUB PaintScreen ()
  DECLARE SUB ShutdownProgram ()
  DECLARE SUB UpdateDateTime ()
  DECLARE SUB WriteRecord (fileName AS STRING)

  DECLARE FUNCTION GetFileName (fileName AS STRING)

'***************************************************************************
'* Declare constants.                                                      *
'***************************************************************************

  CONST PROGRAMNAME = "Label Data Entry"
  CONST PROGRAMVERSION = "1.00"

'***************************************************************************
'* Declare global fields & SHARED arrays.                                  *
'***************************************************************************

  OPTION BASE 1

  DIM SHARED errorFlag AS INTEGER
  DIM SHARED errorNumber AS INTEGER

  DIM screenData(11) AS STRING
 
  DIM screenFields(11) AS FldType

'***************************************************************************
'* Set up event handling.                                                  *
'***************************************************************************

  ON ERROR GOTO error.trap
  ON TIMER(15) GOSUB update.time

'*-------------------------------------------------------------------------*
'* Do mainline processing.                                                 *
'*-------------------------------------------------------------------------*

  Main

'*-------------------------------------------------------------------------*
'* End the program.                                                        *
'*-------------------------------------------------------------------------*

  ShutdownProgram
  END

'***************************************************************************
'* ON ERROR jump label.                                                    *
'***************************************************************************

error.trap:

  ExceptionHandler
  RESUME NEXT

'***************************************************************************
'* ON TIMER jump label.                                                    *
'***************************************************************************

update.time:

  UpdateDateTime
  RETURN

'********************
'* ExceptionHandler *
'***************************************************************************
'*                                                                         *
'* Text . . . :  Handles any exceptions that occur within the program.     *
'*                                                                         *
'* Parameters :  None                                                      *
'*                                                                         *
'* Result . . :  Global field errorNumber is set.                          *
'*                                                                         *
'***************************************************************************
'
SUB ExceptionHandler

'*-------------------------------------------------------------------------*
'* Save the number of the exception that has occurred.                     *
'*-------------------------------------------------------------------------*

  errorNumber = ERR

'*-------------------------------------------------------------------------*
'* If errorFlag has already been set then we are looping so end the program*
'* now.                                                                    *
'*-------------------------------------------------------------------------*
                                                 
  IF errorFlag THEN END

'*-------------------------------------------------------------------------*
'* Set the errorFlag field.  This will make sure that this routine does not*
'* loop.                                                                   *
'*-------------------------------------------------------------------------*

  errFlag = TRUE

'*-------------------------------------------------------------------------*
'* Certain errors will be allowed through & will be handled by the mainline*
'* code.  If this happens then errorFlag is set off.                       *
'*-------------------------------------------------------------------------*

  SELECT CASE errorNumber
  
    CASE 52, 53               ' Bad file/not found
      errorFlag = FALSE
  
    CASE 64                   ' Bad file name
      errorFlag = FALSE

    CASE ELSE

      Box DBLBOX, 6, 26, 13, 54, WHITE, RED, TRUE, TRUE

      DspTxt 7, 28, "Unexpected error occurred", YELLOW, RED
      DspTxt 8, 28, "Number:", YELLOW, RED
      DspTxt 8, 35, STR$(errorNumber), YELLOW, RED
      DspTxt 10, 33, "Program aborted", YELLOW, RED
      DspTxt 12, 27, "<Press any key to continue>", YELLOW, RED

      BoxShadow 6, 26, 13, 54, L.GREY, GREY

      DO
      LOOP WHILE INKEY$ = ""
  
      ShutdownProgram

  END SELECT

END SUB

'***************
'* GetFileName *
'***************************************************************************
'*                                                                         *
'* Text . . . :  Asks the user for the name of the file to use.            *
'*                                                                         *
'* Parameters :  fileName - File name field                                *
'*                                                                         *
'* Result . . :  Key pressed by the user + parameter filled in.            *
'*                                                                         *
'***************************************************************************
'
FUNCTION GetFileName (fileName AS STRING)

'***************************************************************************
'* Declare shared & local fields.                                          *
'***************************************************************************

  DIM nameField AS FldType

'*-------------------------------------------------------------------------*
'* Set up the file name field attributes.                                  *
'*-------------------------------------------------------------------------*
 
  nameField.row = 12
  nameField.col = 45
  nameField.length = 8
  nameField.char = TRUE
  nameField.decs = 0
  nameField.fillChar = CHR$(254)
  nameField.actKey = 0

'*-------------------------------------------------------------------------*
'* Paint the display.                                                      *
'*-------------------------------------------------------------------------*

  Box SGLBOX, 11, 26, 13, 54, WHITE, GREEN, TRUE, TRUE
  BoxShadow 11, 26, 13, 54, L.GREY, GREY

  DspTxt 12, 28, "Data File Name", WHITE, GREEN

  DspTxt 24, 1, SPACE$(80), BLUE, BLUE
  DspTxt 24, 3, "F1=Help   F3=Exit   Enter=Continue", WHITE, BLUE

'*-------------------------------------------------------------------------*
'* Allow the user to enter a file name.                                    *
'*-------------------------------------------------------------------------*

  DO
    InzFld nameField, fileName
   
    WrkFld nameField, fileName, 80, YELLOW, GREEN

'*-------------------------------------------------------------------------*
'* Act on what key has been pressed.                                       *
'*-------------------------------------------------------------------------*

    SELECT CASE nameField.actKey

'*-------------------------------------------------------------------------*
'* Process help displays.                                                  *
'*-------------------------------------------------------------------------*
  
      CASE F1
        HelpText "FILENAME"

      CASE SHFTF1
        HelpText "APPLICATION"

      CASE ALTF12
        HelpText "AUTHOR"

'*-------------------------------------------------------------------------*
'* If F3 pressed then exit this loop.                                      *
'*-------------------------------------------------------------------------*

      CASE F3
        EXIT DO

'*-------------------------------------------------------------------------*
'* Enter has been pressed so try to open the file to see if the file name  *
'* is correct.  If the name is invalid then clear the field and beep.      *
'*-------------------------------------------------------------------------*

      CASE ENTER
        errorNumber = 0

        OPEN "R", 1, fileName

        IF errorNumber = 0 THEN
          fileName = RTRIM$(UCASE$(fileName)) + ".DAT"
          CLOSE 1
          EXIT DO
        ELSE
          BEEP
          fileName = ""
          nameField.ptr = 0
        END IF

    END SELECT

  LOOP

  GetFileName = nameField.actKey

END FUNCTION

'************
'* HelpText *
'***************************************************************************
'*                                                                         *
'* Text . . . :  Application help text handler.                            *
'*                                                                         *
'* Parameters :  helpName - String containing the name of the help to show.*
'*                                                                         *
'* Result . . :  None                                                      *
'*                                                                         *
'***************************************************************************
'
SUB HelpText (helpName AS STRING)

'***************************************************************************
'* Declare shared & local fields.                                          *
'***************************************************************************

  SHARED baseVideoPage AS INTEGER
  SHARED windowVideoPage AS INTEGER

'*-------------------------------------------------------------------------*
'* Copy the current video page to the next & use that one for the help.    *
'*-------------------------------------------------------------------------*

  PCOPY baseVideoPage, windowVideoPage

  SCREEN 0, , windowVideoPage, baseVideoPage

'*-------------------------------------------------------------------------*
'* Display the given help text.                                            *
'*-------------------------------------------------------------------------*

  SELECT CASE helpName

'*-------------------------------------------------------------------------*
'* General application help.                                               *
'*-------------------------------------------------------------------------*

    CASE "APPLICATION"

      Box SGLBOX, 5, 7, 14, 41, WHITE, GREEN, TRUE, TRUE
      BoxShadow 5, 7, 14, 41, L.GREY, GREY

      CenterText 6, 7, 41, PROGRAMNAME, WHITE, GREEN
      CenterText 7, 7, 41, PROGRAMVERSION, WHITE, GREEN
      DspTxt 8, 8, STRING$(33, 196), WHITE, GREEN

      DspTxt 9, 9, "(c) Copyright 1993", WHITE, GREEN
      DspTxt 10, 13, "Blackbird Data Systems Ltd", WHITE, GREEN
      DspTxt 11, 13, "All rights reserved", WHITE, GREEN
      CenterText 13, 7, 41, "<Press ESCAPE to continue>", YELLOW, GREEN

'*-------------------------------------------------------------------------*
'* File name prompt.                                                       *
'*-------------------------------------------------------------------------*

    CASE "FILENAME"

      Box SGLBOX, 5, 3, 15, 34, WHITE, RED, TRUE, TRUE
      BoxShadow 5, 3, 15, 34, L.GREY, GREY

      CenterText 6, 3, 34, PROGRAMNAME, WHITE, RED
      DspTxt 7, 4, STRING$(30, 196), WHITE, RED

      DspTxt 8, 5, "Enter the name of the data", YELLOW, RED
      DspTxt 9, 5, "file you wish to add records", YELLOW, RED
      DspTxt 10, 5, "to.", YELLOW, RED
      DspTxt 12, 5, "Press Enter when done.", YELLOW, RED
      DspTxt 14, 6, "<Press ESCAPE to continue>", YELLOW, RED


'*-------------------------------------------------------------------------*
'* File field display.                                                     *
'*-------------------------------------------------------------------------*

    CASE "FILEFIELD"

      Box SGLBOX, 5, 3, 14, 40, WHITE, RED, TRUE, TRUE
      BoxShadow 5, 3, 14, 40, L.GREY, GREY

      CenterText 6, 3, 34, PROGRAMNAME, WHITE, RED
      DspTxt 7, 4, STRING$(36, 196), WHITE, RED

      DspTxt 8, 5, "Enter the label data into the", YELLOW, RED
      DspTxt 9, 5, "supplied fields.", YELLOW, RED
      DspTxt 11, 5, "Press Enter to add record to file.", YELLOW, RED
      CenterText 13, 3, 40, "<Press ESCAPE to continue>", YELLOW, RED

'*-------------------------------------------------------------------------*
'* Indulgence time.  Show who wrote this program.                          *
'*-------------------------------------------------------------------------*

    CASE "AUTHOR"

      Box DBLBOX, 6, 6, 13, 35, WHITE, CYAN, TRUE, TRUE
      BoxShadow 6, 6, 13, 35, L.GREY, GREY

      CenterText 7, 6, 35, PROGRAMNAME, WHITE, CYAN
      DspTxt 8, 7, STRING$(28, 196), WHITE, CYAN

      DspTxt 9, 8, "Program written by:", YELLOW, CYAN
      DspTxt 10, 12, AUTHOR, BLACK, CYAN
      DspTxt 12, 8, "<Press ESCAPE to continue>", YELLOW, CYAN

    CASE ELSE

  END SELECT

'*-------------------------------------------------------------------------*
'* Show the video page.                                                    *
'*-------------------------------------------------------------------------*

  SCREEN 0, , windowVideoPage, windowVideoPage

'*-------------------------------------------------------------------------*
'* Loop around until escape is pressed.                                    *
'*-------------------------------------------------------------------------*

  DO
  LOOP UNTIL WaitKey = ESCAPE

'*-------------------------------------------------------------------------*
'* Reset the video page & update the date/time on the base page.           *
'*-------------------------------------------------------------------------*

  SCREEN 0, , baseVideoPage, windowVideoPage
  UpdateDateTime
  SCREEN 0, , baseVideoPage, baseVideoPage

END SUB

'*********************
'* InitialiseProgram *
'***************************************************************************
'*                                                                         *
'* Text . . . :  Application initialisation.  Collect all necessary start  *
'*               up data & set up the screen.                              *
'*                                                                         *
'* Parameters :  None                                                      *
'*                                                                         *
'* Result . . :  Sets some shared videopage fields for use when windowing. *
'*                                                                         *
'***************************************************************************
'
SUB InitialiseProgram

'***************************************************************************
'* Declare shared & local fields.                                          *
'***************************************************************************
 
  SHARED baseVideoPage AS INTEGER
  SHARED entryVideoPage AS INTEGER
  SHARED windowVideoPage AS INTEGER

  SHARED screenFields() AS FldType

  DIM videoMode AS INTEGER
  DIM noOfColumns AS INTEGER

'*-------------------------------------------------------------------------*
'* Retrieve the current video page for use by all windowing functions.  The*
'* base page for this application will be one more than this & all windows *
'* are assumed to only use the next page up from the base page.            *
'*-------------------------------------------------------------------------*

  VideoState videoMode, noOfColumns, entryVideoPage

  baseVideoPage = (entryVideoPage + 1)
  windowVideoPage = (baseVideoPage + 1)

'*-------------------------------------------------------------------------*
'* Paint the main screen.                                                  *
'*-------------------------------------------------------------------------*

  PaintScreen

'*-------------------------------------------------------------------------*
'* Set up the field attributes.                                            *
'*-------------------------------------------------------------------------*

' Product code

  screenFields(1).row = 9
  screenFields(1).col = 38
  screenFields(1).length = 15
  screenFields(1).char = TRUE
  screenFields(1).decs = 0
  screenFields(1).fillChar = CHR$(254)
  screenFields(1).actKey = 0

' Order number

  screenFields(2).row = 10
  screenFields(2).col = 38
  screenFields(2).length = 8
  screenFields(2).char = TRUE
  screenFields(2).decs = 0
  screenFields(2).fillChar = CHR$(254)
  screenFields(2).actKey = 0

' Drum number

  screenFields(3).row = 11
  screenFields(3).col = 38
  screenFields(3).length = 9
  screenFields(3).char = TRUE
  screenFields(3).decs = 0
  screenFields(3).fillChar = CHR$(254)
  screenFields(3).actKey = 0

' Metres

  screenFields(4).row = 12
  screenFields(4).col = 38
  screenFields(4).length = 4
  screenFields(4).char = TRUE
  screenFields(4).decs = 0
  screenFields(4).fillChar = CHR$(254)
  screenFields(4).actKey = 0

' NTV Indicator

  screenFields(5).row = 13
  screenFields(5).col = 38
  screenFields(5).length = 1
  screenFields(5).char = TRUE
  screenFields(5).decs = 0
  screenFields(5).fillChar = CHR$(254)
  screenFields(5).actKey = 0

' Product Description #1

  screenFields(6).row = 14
  screenFields(6).col = 38
  screenFields(6).length = 30
  screenFields(6).char = TRUE
  screenFields(6).decs = 0
  screenFields(6).fillChar = CHR$(254)
  screenFields(6).actKey = 0

' Product Description #2

  screenFields(7).row = 15
  screenFields(7).col = 38
  screenFields(7).length = 30
  screenFields(7).char = TRUE
  screenFields(7).decs = 0
  screenFields(7).fillChar = CHR$(254)
  screenFields(7).actKey = 0

' Weight (kg)

  screenFields(8).row = 16
  screenFields(8).col = 38
  screenFields(8).length = 5
  screenFields(8).char = TRUE
  screenFields(8).decs = 0
  screenFields(8).fillChar = CHR$(254)
  screenFields(8).actKey = 0

' Cable Description

  screenFields(9).row = 17
  screenFields(9).col = 38
  screenFields(9).length = 10
  screenFields(9).char = TRUE
  screenFields(9).decs = 0
  screenFields(9).fillChar = CHR$(254)
  screenFields(9).actKey = 0

' Pairage Description

  screenFields(10).row = 18
  screenFields(10).col = 38
  screenFields(10).length = 13
  screenFields(10).char = TRUE
  screenFields(10).decs = 0
  screenFields(10).fillChar = CHR$(254)
  screenFields(10).actKey = 0

' Date (dd/mm/yy)

  screenFields(11).row = 19
  screenFields(11).col = 38
  screenFields(11).length = 8
  screenFields(11).char = TRUE
  screenFields(11).decs = 0
  screenFields(11).fillChar = CHR$(254)
  screenFields(11).actKey = 0

'*-------------------------------------------------------------------------*
'* Initialise global fields & set on event handling.                       *
'*-------------------------------------------------------------------------*

  errorFlag = FALSE
  errorNumber = 0

  TIMER ON

END SUB

'********
'* Main *
'***************************************************************************
'*                                                                         *
'* Text . . . :  Main line program code.                                   *
'*                                                                         *
'* Parameters :  None                                                      *
'*                                                                         *
'* Result . . :  None                                                      *
'*                                                                         *
'***************************************************************************
'
SUB Main

'***************************************************************************
'* Declare shared & local fields.                                          *
'***************************************************************************

  SHARED dataFile AS INTEGER

  SHARED screenData() AS STRING

  SHARED screenFields() AS FldType

  DIM fldPtr AS INTEGER
  DIM genVar AS INTEGER
  DIM result AS INTEGER

  DIM recordCount AS LONG

  DIM fileName AS STRING

  DIM labelRecord AS LabelType

'*-------------------------------------------------------------------------*
'* Perform initialisation.                                                 *
'*-------------------------------------------------------------------------*

  InitialiseProgram

'*-------------------------------------------------------------------------*
'* Get the file to write data into.                                        *
'*-------------------------------------------------------------------------*

  result = GetFileName(fileName)
 
  IF result = F3 THEN EXIT SUB

  dataFile = FREEFILE

  OPEN "R", dataFile, fileName, LEN(labelRecord)

'*-------------------------------------------------------------------------*
'* Paint the box for the fields.                                           *
'*-------------------------------------------------------------------------*
 
  Box SGLBOX, 6, 12, 21, 69, WHITE, GREEN, TRUE, TRUE
  BoxShadow 6, 12, 21, 69, L.GREY, GREY

  CenterText 7, 12, 69, "File: " + fileName, WHITE, GREEN
  DspTxt 8, 13, STRING$(56, 196), WHITE, GREEN
 
  DspTxt 9, 14, "Product Code  . . . .", YELLOW, GREEN
  DspTxt 10, 14, "Order Number  . . . .", YELLOW, GREEN
  DspTxt 11, 14, "Drum Number . . . . .", YELLOW, GREEN
  DspTxt 12, 14, "Metres  . . . . . . .", YELLOW, GREEN
  DspTxt 13, 14, "NTV Indicator . . . .", YELLOW, GREEN
  DspTxt 14, 14, "Product Description .", YELLOW, GREEN
  DspTxt 16, 14, "Weight (kg) . . . . .", YELLOW, GREEN
  DspTxt 17, 14, "Cable Description . .", YELLOW, GREEN
  DspTxt 18, 14, "Pairage Description .", YELLOW, GREEN
  DspTxt 19, 14, "Date (dd/mm/yy) . . .", YELLOW, GREEN

  DspTxt 24, 1, SPACE$(80), BLUE, BLUE
  DspTxt 24, 3, "F1=Help   F3=Exit   Enter=Add record", WHITE, BLUE
 
'*-------------------------------------------------------------------------*
'* Loop around until the user presses F3.                                  *
'*-------------------------------------------------------------------------*

  DO

'*-------------------------------------------------------------------------*
'* Display the current number of records in the file.                      *
'*-------------------------------------------------------------------------*

  recordCount = (LOF(dataFile) / LEN(labelRecord))

  DspTxt 24, 50, "Records in file:" + STR$(recordCount), WHITE, BLUE

'*-------------------------------------------------------------------------*
'* Initialise all screen fields.                                           *
'*-------------------------------------------------------------------------*

    FOR genVar = 1 TO 11
      screenFields(genVar).ptr = 0
      screenData(genVar) = ""
      InzFld screenFields(genVar), screenData(genVar)
    NEXT genVar

    fldPtr = 1

'*-------------------------------------------------------------------------*
'* Show the fields & allow the user to enter data into them.               *
'*-------------------------------------------------------------------------*

    FOR genVar = 1 TO 11
      DspFld screenFields(genVar), screenData(genVar), 80, BLACK, GREEN
    NEXT genVar

    DO
      WrkFld screenFields(fldPtr), screenData(fldPtr), 80, BLACK, GREEN

      SELECT CASE screenFields(fldPtr).actKey

'*-------------------------------------------------------------------------*
'* Process help displays.                                                  *
'*-------------------------------------------------------------------------*
 
        CASE F1
          HelpText "FILEFIELD"

        CASE SHFTF1
          HelpText "APPLICATION"

        CASE ALTF12
          HelpText "AUTHOR"

'*-------------------------------------------------------------------------*
'* If Enter of F3 pressed then exit this loop.                             *
'*-------------------------------------------------------------------------*
     
        CASE F3, ENTER
          EXIT DO

'*-------------------------------------------------------------------------*
'* Process tabbing between fields.                                         *
'*-------------------------------------------------------------------------*

        CASE FWDTAB
          fldPtr = fldPtr + 1
          IF fldPtr = 12 THEN fldPtr = 1

        CASE BACKTAB
          fldPtr = fldPtr - 1
          IF fldPtr = 0 THEN fldPtr = 11
        
        CASE ELSE

      END SELECT

    LOOP

    IF screenFields(fldPtr).actKey = ENTER THEN
      WriteRecord fileName
    END IF

  LOOP UNTIL screenFields(fldPtr).actKey = F3

  CLOSE

END SUB

'***************
'* PaintScreen *
'***************************************************************************
'*                                                                         *
'* Text . . . :  Paints the main screen.                                   *
'*                                                                         *
'* Parameters :  None                                                      *
'*                                                                         *
'* Result . . :  None                                                      *
'*                                                                         *
'***************************************************************************
'
SUB PaintScreen

'***************************************************************************
'* Declare shared & local fields.                                          *
'***************************************************************************

  SHARED baseVideoPage AS INTEGER
  SHARED entryVideoPage AS INTEGER

'*-------------------------------------------------------------------------*
'* Move onto the program base video page & clear it.                       *
'*-------------------------------------------------------------------------*

  SCREEN 0, , baseVideoPage, entryVideoPage
 
  COLOR L.GREY, BLACK
  CLS

'*-------------------------------------------------------------------------*
'* Set up the main part of the display.                                    *
'*-------------------------------------------------------------------------*

  Box DBLBOX, 1, 1, 3, 80, WHITE, BLUE, TRUE, TRUE
  CenterText 2, 1, 80, PROGRAMNAME, WHITE, BLUE
  DspTxt 4, 2, STRING$(78, 223), GREY, L.GREY
  Box SGLBOX, 5, 2, 24, 79, L.GREY, L.GREY, TRUE, TRUE
 
  DspTxt 24, 1, SPACE$(80), BLUE, BLUE
  DspTxt 25, 2, STRING$(78, 223), GREY, L.GREY
 
  UpdateDateTime

'*-------------------------------------------------------------------------*
'* Now allow the base screen to be seen.                                   *
'*-------------------------------------------------------------------------*

  SCREEN 0, , baseVideoPage, baseVideoPage

END SUB

'*******************
'* ShutdownProgram *
'***************************************************************************
'*                                                                         *
'* Text . . . :  Shuts down the application safely.                        *
'*                                                                         *
'* Parameters :  None                                                      *
'*                                                                         *
'* Result . . :  None                                                      *
'*                                                                         *
'***************************************************************************
'
SUB ShutdownProgram

'***************************************************************************
'* Declare shared & local fields.                                          *
'***************************************************************************

  SHARED baseVideoPage AS INTEGER
  SHARED entryVideoPage AS INTEGER

'*-------------------------------------------------------------------------*
'* Clear down the base video page & return to using the entry video page.  *
'*-------------------------------------------------------------------------*

  COLOR L.GREY, BLACK
  CLS

  SCREEN 0, , entryVideoPage, entryVideoPage

'*-------------------------------------------------------------------------*
'* End the run.                                                            *
'*-------------------------------------------------------------------------*

  END

END SUB

'******************
'* UpdateDateTime *
'***************************************************************************
'*                                                                         *
'* Text . . . :  Updates the date & time that are on the screen.           *
'*                                                                         *
'* Parameters :  None                                                      *
'*                                                                         *
'* Result . . :  None                                                      *
'*                                                                         *
'***************************************************************************
'
SUB UpdateDateTime

'*-------------------------------------------------------------------------*
'* Update the on screen date and time.                                     *
'*-------------------------------------------------------------------------*

  DspTxt 2, 3, GetDate$("DMY"), YELLOW, BLUE
  DspTxt 2, 74, LEFT$(TIME$, 5), YELLOW, BLUE

END SUB

'***************
'* WriteRecord *
'***************************************************************************
'*                                                                         *
'* Text . . . :  Writes a record out to the given data file.               *
'*                                                                         *
'* Parameters :  fileName - File to write to.                              *
'*                                                                         *
'* Result . . :  A record is written !!!                                   *
'*                                                                         *
'***************************************************************************
'
SUB WriteRecord (fileName AS STRING)

'***************************************************************************
'* Declare shared & local fields.                                          *
'***************************************************************************
 
  SHARED dataFile AS INTEGER

  SHARED screenData() AS STRING
 
  DIM fileLen AS LONG

  DIM fileRecord AS LabelType

'*-------------------------------------------------------------------------*
'* Move screen data into the file record structure.                        *
'*-------------------------------------------------------------------------*

  fileRecord.prdCode = screenData(1)
  fileRecord.ordNo = screenData(2)
  fileRecord.drumNo = screenData(3)
  fileRecord.metres = screenData(4)
  fileRecord.NTVFlag = screenData(5)
  fileRecord.prdDsc1 = screenData(6)
  fileRecord.prdDsc2 = screenData(7)
  fileRecord.weight = screenData(8)
  fileRecord.cableDsc = screenData(9)
  fileRecord.pairageDsc = screenData(10)
  fileRecord.date = screenData(11)

'*-------------------------------------------------------------------------*
'* Find out the length of the file & set the file pointer to be the next   *
'* available record number.                                                *
'*-------------------------------------------------------------------------*

  fileLen = LOF(dataFile)
  SEEK dataFile, (fileLen / LEN(fileRecord)) + 1

  PUT #dataFile, , fileRecord

END SUB

