{*******************************************************}
{ This program demonstrates how to create an ownerdraw  }
{ listbox. There may be better ways to do it, but here  }
{ at least is ONE way...                                }
{                                                       }
{ It displays all the available disk drive letters, with}
{ a disk icon next to each one.                         }
{                                                       }
{ I am greatful to Kurt Barthelmess and Mike Scott      }
{ for technical assistance, without which, I never      }
{ could have figured out how to do this.                }
{                                                       }
{ If you can figure out how to use a different kind of  }
{ disk icon for floppy drives, hard drives, ram drives  }
{ and networked drives, please let me know.             }
{ Contact me on Compuserve at 73557,203. - Steve Leonard}
{*******************************************************}

program LBTest;
{$R LBDEMO.RES}
uses WinTypes, WinProcs, OWindows, ODialogs, strings;

const
  id_LB1 = 101;
  id_bitmap = 103;
type
  TestApplication = object(TApplication)
    procedure InitMainWindow; virtual;
  end;

  PTestWindow = ^TestWindow;

  TestWindow = object(TWindow)
    HDiskbmp : Hbitmap;
    LB1: PListBox;
    constructor Init(AParent: PWindowsObject; ATitle: PChar);
    destructor done; virtual;
    procedure SetupWindow; virtual;
    procedure wmmeasureitem(var Msg:TMessage); virtual wm_first + wm_measureitem;
    procedure wmdrawitem(var Msg:TMessage); virtual wm_first+wm_drawitem;
    procedure DrawItem( CtrlID         : integer ;
               DrawItemStruct : PDrawItemStruct ) ; virtual;
    procedure HandleListBoxMsg(var Msg: TMessage);
      virtual id_First + id_LB1;
  end;

procedure TestWindow.wmMeasureItem( var Msg : TMessage ) ;

var DC : HDC ;
    TextMetrics : TTextMetric ;
    result      : boolean ;

begin
  DC := GetDC( HWindow ) ;
  result := GetTextMetrics( DC, TextMetrics ) ;
  ReleaseDC( HWindow, DC ) ;
  with TextMetrics, PMeasureItemStruct( Msg.LParam )^ do if result then
    ItemHeight := tmHeight + tmExternalLeading else ItemHeight := 20 ;
end ;

procedure TestWindow.DrawItem( CtrlID         : integer ;
                               DrawItemStruct : PDrawItemStruct ) ;
var
  Memdc : HDC;
  MemBM: TBitmap;
  oldbitmap : HBitmap;
  i : byte;
  name : array[0..12] of char;
begin
  with DrawItemStruct^ do with rcItem do begin
    SetBkMode( hDC, Transparent ) ;
    lb1^.getstring(name,itemid);
    for i := 0 to 12 do
    begin
     {  name[i] := upcase(name[i]);}
       name[4] := ':';
       if name[i] in ['[',']','-']
          then name[i] := ' ';
     end;
    TextOut( hDC, Left+30, Top, name, strlen(name)) ;

   memdc := createcompatibledc(hdc);
   oldbitmap := selectobject(memdc,HDiskBMP);
   GetObject(HDiskBMP, Sizeof(MemBM), @MemBM);
   BitBlt(hDC, Left+10,Top+4, MemBM.bmWidth, MemBM.bmHeight-9, MemDC, 0, 0,srccopy);
   selectobject(memdc,oldbitmap);
   deletedc(memdc);

  end ;
end ;

procedure TestWindow.wmDrawItem( var Msg : TMessage ) ;

var Selected    : boolean ;
    OldColor    : TColorRef ;
    OldBkGnd    : TColorRef ;
    Brush       : HBrush ;

begin
  with PDrawItemStruct( Msg.LParam )^ do begin

    if ( itemAction and ( oda_DrawEntire or oda_Select ) ) <> 0 then begin

      { set colors depending on selection state }
      Selected := ( itemState and ods_Selected ) <> 0 ;
      if Selected then begin
        OldColor  := SetTextColor( hDC, paletteindex(4));
        OldBkGnd  := SetBkColor( hDC, paletteindex(2));
        Brush     := CreateSolidBrush( paletteindex(2) ) ;
      end else
        Brush :=  CreateSolidBrush( paletteindex(15) ) ;
      FillRect( hDC, rcItem, Brush ) ;
      DeleteObject( Brush ) ;

      DrawItem( CtlID, PDrawItemStruct( Msg.LParam ) ) ;

      { if selected, reset to original colors }
      if Selected then begin
        SetTextColor( hDC, OldColor ) ;
        SetBkColor( hDC, OldBkGnd ) ;
      end ;
    end ;
  end ;
  DefWndProc( Msg ) ;
end ;


constructor TestWindow.Init(AParent: PWindowsObject; ATitle: PChar);
begin
  inherited Init(AParent, ATitle);
  LB1 := New(PListBox, Init(@Self, id_LB1, 20, 20, 130, 100));

  lb1^.attr.style := ( lb1^.attr.style or lbs_ownerdrawfixed or lbs_hasstrings )
                       and not lbs_Sort ;

  HDiskBMP := loadbitmap(Hinstance,pchar(id_bitmap));
end;

destructor TestWindow.done;
begin
  deleteobject(HDiskBMP);
  inherited done;
end;

procedure TestWindow.SetupWindow;
begin
  inherited SetupWindow;

  { Here's how to manually add items to test with }
  {
  LB1^.AddString('C:');
  LB1^.AddString('D:');
  LB1^.AddString('E:');
  }
  SendMessage(LB1^.HWindow, lb_Dir, $C000, longint(pchar('*.*')));
end;

procedure TestWindow.HandleListBoxMsg(var Msg: TMessage);
var
  Idx: Integer;
  ItemText: array[0..20] of Char;
  w1 : word;
  l1, l2 :longint;
  s1 : string;
begin
  if Msg.LParamHi = lbn_SelChange then
  begin
    Idx := LB1^.GetSelIndex;
    if LB1^.GetStringLen(Idx) < SizeOf(ItemText) then
    begin
      l1 := 0;
      w1 := idx;
{      l2 := sendmessage(LB1^.Hwindow,lb_getitemdata,w1,l1);}
      lb1^.getstring(itemtext,w1);
      MessageBox(HWindow, itemtext, 'You selected:', mb_OK);
    end;
  end
  else DefWndProc(Msg);
end;

procedure TestApplication.InitMainWindow;
begin
  MainWindow := New(PTestWindow, Init(nil, 'List Box Tester'));
end;

var
  TestApp : TestApplication;

begin
  TestApp.Init('LBoxTest');
  TestApp.Run;
  TestApp.Done;
end.

