Unit LVI;


{ LVI UNIT VERSION 1.0 BY TOBIN FRICKE }

(* This unit provides a means of using the Last Video Interface (LVI).     *)
(* If you find this unit usefull, a $1.00 contibution is requested.        *)

(* Reaching the Author:                                                    *)

(* CompuServe: 76660,3110                                                  *)
(* Cleveland Freenet: dr261                                                *)
(* Internet: dr261@cleveland.freenet.edu                                   *)
(* Snail Mail: Tobin Fricke, 25271 Arion Way, Mission Viejo, CA 92691-3702 *)


(* This may be distributed freely in its original, unmodified form only.   *)
(* No fee may be charged for the distribution of this program, except for  *)
(* a modest per disk fee if that applies.                                  *)


(* Special thanks to John C. Bushnell, the designer of LVI, for a great    *)
(* new standard.                                                           *)


(* What is LVI?  It's a new standard to replace ANSI.  It uses binary, and *)
(* can be twice as fast as ANSI.  It provides means for using windows,     *)
(* scrolling, and many other great screen control procedures.              *)


{$IFDEF DPMI}
{$D LVI Unit by Tobin Fricke}
{$ENDIF}

Interface

Procedure TextColor(i:Byte);
Procedure BackgroundColor(i:Byte);
Procedure GotoXY(X,Y:Byte);
Procedure WriteChar(i:Byte);
Procedure SaveCursorPos(Slot:Byte);
Procedure RestoreCursorPos(slot:Byte);
Procedure Colorize(length:byte);
Procedure ColorizeWindow;
Procedure FillWindow(FillChar:Byte);
Procedure RepeatChar(RepeatChar,RepeatTimes:Byte);
Procedure RepeatCharDownward(RepeatChar,RepeatTimes:Byte);
Procedure DefineMacro(Macro:String);
Procedure StampMacro;
Procedure WriteStringDownward(S:String);
Procedure ResetWindowSize;
Procedure SetWindowCoordinates(x1,y1,x2,y2:byte);
Procedure ScrollWindowLeftOnce;
Procedure ScrollWindowRightOnce;
Procedure ScrollWindowDownwardOnce;
Procedure ScrollWindowUpwardOnce;
Procedure ScrollWindowLeft(times:byte);
Procedure ScrollWindowRight(times:byte);
Procedure ScrollWindowUpward(times:byte);
Procedure ScrollWindowDownward(times:byte);
Procedure SaveWindowContents(Slot:byte);
Procedure RestoreWindowContents(Slot:byte);
Procedure RestoreWindowContentsAndClearSlot(slot:byte);
Procedure LP;

Implementation

{ Escape charactors:   }
{ #27 = ANSI CODE      }
{ #28 = LVI CODE       }
{ #29 = LVI COLOR CODE }

Var F,B:Byte;

Procedure LP; {List Procedures}
(* Just ignore this procedure.  It's for if you don't have the source code. *)
begin
 Writeln('Unit LVI;                                                                    ');
 Writeln('                                                                             ');
 Writeln('                                                                             ');
 Writeln('{ LVI UNIT VERSION 1.0 BY TOBIN FRICKE }                                     ');
 Writeln('                                                                             ');
 Writeln('(* This unit provides a means of using the Last Video Interface (LVI).     *)');
 Writeln('(* If you find this unit usefull, a $1.00 contibution is requested.        *)');
 Writeln('                                                                             ');
 Writeln('(* Reaching the Author:                                                    *)');
 Writeln('                                                                             ');
 Writeln('(* CompuServe: 76660,3110                                                  *)');
 Writeln('(* Cleveland Freenet: dr261                                                *)');
 Writeln('(* Internet: dr261@cleveland.freenet.edu                                   *)');
 Writeln('(* Snail Mail: Tobin Fricke, 25271 Arion Way, Mission Viejo, CA 92691-3702 *)');
 Writeln('                                                                             ');
 Writeln('                                                                             ');
 Writeln('(* This may be distributed freely in its original, unmodified form only.   *)');
 Writeln('(* No fee may be charged for the distribution of this program, except for  *)');
 Writeln('(* a modest per disk fee if that applies.                                  *)');
 Writeln('                                                                             ');
 Readln;
 Writeln('                                                                             ');
 Writeln('(* Special thanks to John C. Bushnell, the designer of LVI, for a great    *)');
 Writeln('(* new standard.                                                           *)');
 Writeln('                                                                             ');
 Writeln('                                                                             ');
 Writeln('(* What is LVI?  It''s a new standard to replace ANSI.  It uses binary, and *');
 Writeln('(* can be twice as fast as ANSI.  It provides means for using windows,     *)');
 Writeln('(* scrolling, and many other great screen control procedures.              *)');
 Writeln('                                                                             ');
 Writeln('                                                                             ');
 Writeln('{$IFDEF DPMI}                                                                ');
 Writeln('{$D LVI Unit by Tobin Fricke}                                                ');
 Writeln('{$ENDIF}                                                                     ');
 Writeln('                                                                             ');
 Writeln('Interface                                                                    ');
 Writeln('                                                                             ');
 Writeln('Procedure TextColor(i:Byte);                                                 ');
 Writeln('Procedure BackgroundColor(i:Byte);                                           ');
 Writeln('Procedure GotoXY(X,Y:Byte);                                                  ');
 Readln;
 Writeln('Procedure WriteChar(i:Byte);                                                 ');
 Writeln('Procedure SaveCursorPos(Slot:Byte);                                          ');
 Writeln('Procedure RestoreCursorPos(slot:Byte);                                       ');
 Writeln('Procedure Colorize(length:byte);                                             ');
 Writeln('Procedure ColorizeWindow;                                                    ');
 Writeln('Procedure FillWindow(FillChar:Byte);                                         ');
 Writeln('Procedure RepeatChar(RepeatChar,RepeatTimes:Byte);                           ');
 Writeln('Procedure RepeatCharDownward(RepeatChar,RepeatTimes:Byte);                   ');
 Writeln('Procedure DefineMacro(Macro:String);                                         ');
 Writeln('Procedure StampMacro;                                                        ');
 Writeln('Procedure WriteStringDownward(S:String);                                     ');
 Writeln('Procedure ResetWindowSize;                                                   ');
 Writeln('Procedure SetWindowCoordinates(x1,y1,x2,y2:byte);                            ');
 Writeln('Procedure ScrollWindowLeftOnce;                                              ');
 Writeln('Procedure ScrollWindowRightOnce;                                             ');
 Writeln('Procedure ScrollWindowDownwardOnce;                                          ');
 Writeln('Procedure ScrollWindowUpwardOnce;                                            ');
 Writeln('Procedure ScrollWindowLeft(times:byte);                                      ');
 Writeln('Procedure ScrollWindowRight(times:byte);                                     ');
 Readln;
 Writeln('Procedure ScrollWindowUpward(times:byte);                                    ');
 Writeln('Procedure ScrollWindowDownward(times:byte);                                  ');
 Writeln('Procedure SaveWindowContents(Slot:byte);                                     ');
 Writeln('Procedure RestoreWindowContents(Slot:byte);                                  ');
 Writeln('Procedure RestoreWindowContentsAndClearSlot(slot:byte);                      ');
 Writeln('Procedure LP;                                                                ');
 Writeln;
 Readln;
 end;

Procedure SetColor;
{ This procedure is called by TextColor and BackgroundColor }
begin
 Write(#29+Char(F+(B*16)));
end;

Procedure TextColor(i:Byte);
{ This changes the Text (forground) color to i              }
begin
 F:=I;
 SetColor;
end;

Procedure BackgroundColor(i:Byte);
{ This changes the Background color to i                    }
begin
 B:=I;
 SetColor;
end;


Procedure WriteChar(i:Byte);
{ Just like the pascal Write(char(i)), excpet it does it remotely. }
begin
 Write(#28+#0+Char(i));
end;


Procedure SaveCursorPos(Slot:Byte);
{ This saves the cursor position into slot slot.  slot is from 0 to 7}
begin
 Write(#28+'s'+char(Slot));
end;

Procedure RestoreCursorPos(Slot:Byte);
{ Restores the cursor position from slot slot.  See SaveCursorPos    }
begin
 Write(#28+'u'+char(Slot));
end;

Procedure Colorize(length:byte);
{ Colorizes length charactors to the current color selection         }
begin
 Write(#28+'='+char(Length));
end;

Procedure ColorizeWindow;
{ Changes the entire window to the current colors                    }
begin
 Write(#28+#176);
end;

Procedure FillWindow(FillChar:Byte);
{ Fills the window with the current color and charactor FillChar     }
begin
 Write(#28+#219+char(FillChar));
end;

Procedure RepeatChar(RepeatChar,RepeatTimes:Byte);
{ Repeats charactor RepeatChar RepeatTimes times. }
begin
 Write(#28+#196+char(RepeatChar)+char(RepeatTimes));
end;


Procedure RepeatCharDownward(RepeatChar,RepeatTimes:Byte);
{Same as RepearChar, but vertical }
begin
 Write(#28+#179+char(RepeatChar)+char(RepeatTimes));
end;


Procedure DefineMacro(Macro:String);
{ Defines a macro }
begin
 Write(#28+#254+Char(Length(Macro))+Macro);
end;

Procedure StampMacro;
begin
 Write(#28+'*');
end;

Procedure WriteStringDownward(S:String);
{ Like Writeln, but Vertical }
begin
 Write(#28+'|'+char(length(s))+S);
end;

Procedure ResetWindowSize;
{ Resets the window and makes it as big as possible }
begin
 Write(#28+'W');
end;

Procedure SetWindowCoordinates(x1,y1,x2,y2:byte);
{ Sets the window's coordinates }
begin
 Write(#28+'w'+char(x1)+char(y1)+char(x2)+char(y2));
end;


Procedure ScrollWindowLeftOnce;
{ Self explanatory, I hope }
begin
 Write(#28+'<');
end;

Procedure ScrollWindowLeft(times:byte);
begin
 Write(#28+'{'+char(Times));
end;

Procedure ScrollWindowRightOnce;
begin
 Write(#28+'>');
end;

Procedure ScrollWindowRight(times:byte);
begin
 Write(#28+'}'+char(times));
end;

Procedure ScrollWindowUpwardOnce;
begin
 Write(#28+#30);
end;

Procedure ScrollWindowUpward(times:byte);
begin
 Write(#28+#24+char(Times));
end;

Procedure ScrollWindowDownwardOnce;
begin
 Write(#28+#31);
end;

Procedure ScrollWindowDownward(times:byte);
begin
 Write(#28+#25+char(times));
end;

Procedure SaveWindowContents(Slot:byte);
{ Saves the contents of the window into slot slot.  Slot is from 0 to 2. }
begin
 Write(#28+#236+Char(Slot));
end;

Procedure RestoreWindowContentsAndClearSlot(slot:byte);
begin
 Write(#28+#237+Char(slot));
end;

Procedure RestoreWindowContents(Slot:byte);
begin
 Write(#28+#238+Char(Slot));
end;

Procedure GotoXY(X,Y:Byte);
{ This is just like in pascal. It is not LVI, but it is ANSI.            }
var SX,SY:string;
begin
 Str(X,SX);
 Str(Y,SY);
 write(#27+'['+SY+';'+SX+'H');
end;


Begin {Initialization}
 F:=7;{Set initial forground color to 7 (gray) }
 B:=0;{Set initial background color to 0 (black}
end.