/*
 *       ͻ
 *              Chi Programmer's Editor, Borland C++ 3.1       
 *                     
 *           Copyright (C) 1990,1991,1993                      
 *           Author: Fook H. Eng, 201 West Hilliard Avenue     
 *                   Monterey Park, CA 91754, 818 572-9220     
 *                                                             
 *       ͼ
 */
#include    <stdio.h>
#include    <stdlib.h>
#include    <ctype.h>
#include    <string.h>
#include    <conio.h>
#include    <dos.h>
#include    "vartypes.h"
#include    "utils.h"
#include    "memory.h"
#include    "keys.h"
#include    "cursors.h"
#include    "windows.h"
#include    "qcmd.h"
#include    "screen.h"
#include    "blocks.h"

static  void  cpwindowdelete(void);
static  void  setskip(pchar);
static  int   match(char , char );
static  void  filterstring(pchar);
static  void  do_replace(void);
static  void  blockreplace(void);

void    deletetextright()
{
    int     i;
    pLineDesc cl = curwin->curline;

    if (curwin->colno < cl->bufflen)
        for (i = curwin->colno; i <= cl->bufflen; i++)
            cl->ptxt[i-1] = ' ';
}

void    windowdeletetext()
{
    pLineDesc p, q;

    p = curwin->topline;

    while (p->paft != NULL) p = p->paft;

    while (p != NULL) {
        q = p;
        p = p->pfwd;
        freetxtdes(q);
    }

    strcpy(curwin->filename, "nofile");
    curwin->topline = maktxtdes(1);
    curwin->curline = curwin->topline;
    curwin->colno = 1;
    realign();
}

static  void  cpwindowdelete()
{
    sprintf(cmdline2, "delete %s from window? [y/n]", curwin->filename);

    if (getyesno(cmdline2)) windowdeletetext();

    appcmdline("");
}

static char skip[0x100];

static void setskip(pchar pattern)
{
    int     ch;
    int     i, lenp;

    lenp = strlen(pattern);

    for (i = 0; i < 0x100; i++) skip[i] = lenp;

    for (i = 1; i <= lenp; i++) {
        skip[pattern[i-1]] = lenp - i;
        if (igncase) {
            if (islower(pattern[i-1]))
                skip[toupper(pattern[i-1])] = lenp - i;
            else if (isupper(pattern[i-1]))
                skip[tolower(pattern[i-1])] = lenp - i;
        }
    }
}

static int  match(char ch1, char ch2)
{
    return (igncase) ? (toupper(ch1) == toupper(ch2)) : (ch1 == ch2);
}


/***
    The Moore-Boyer algorithm from Robert Sedgewick's Algorithm p251
                                                                   ***/
pLineDesc scanforward(pLineDesc p, int * col)
{
    int         found, m, i, j;

    m = j = strlen(searchstr);
    i = *col + m - 1;
    found = FALSE;

    setskip(searchstr);

    do {

        while (i <= p->bufflen && !found) {

            if (match(p->ptxt[i-1], searchstr[j-1]))  {   i--; j--;  }
            else  {  i += max(skip[p->ptxt[i-1]], m-j+1);  j = m;  }

            if (j < 1) {
                found = TRUE;
                if (wholeword) {
                    if (i >= 1)  found = !isalpha(p->ptxt[i-1]);
                    if (found && i+m+1 <= p->bufflen)  found = !isalpha(p->ptxt[i+m]);
                }
            }
        }

        if (found)  {  *col = i + 1;  return p;  }
        else {
            i = j = m;
            p = scanningfwd ? p->pfwd : p->paft;
        }

    } while (!found && p != NULL);

    return NULL;
}

static void filterstring(pchar st)
{
    pchar  pslash;         /* pointer to slash in input string */

    igncase = wholeword = FALSE;
    scanningfwd = TRUE;

    if ((pslash = strchr(st, '/')) != NULL) {
        igncase     = strpbrk(pslash, "iI") != NULL;
        wholeword   = strpbrk(pslash, "wW") != NULL;
        scanningfwd = strpbrk(pslash, "rR") == NULL;
        if (!scanningfwd) curwin->colno = 1;
        *pslash = '\0';
        if (st[strlen(st) - 1] == ' ') *(--pslash) = '\0';
    }

    if (strlen(st) != 0)
        strcpy (searchstr, st);
}


void    find()
{
    pLineDesc p;
    pLineDesc q;
    int        i;

    if ((p = scanforward(curwin->curline, &curwin->colno)) != NULL) {
        curwin->curline = p;
        curwin->colno += strlen(searchstr);

        q = curwin->topline;
        for (i = curwin->firstlineno;;) {       /* compare with replace() */
            if (q == p) break;
            if (q->pfwd == NULL || ++i == curwin->lastlineno) break;
            q = q->pfwd;
        }

        if (q != p) {
            curwin->topline = p;
            realign();
            linehalfdown();
        }
        realign();
        refresh_screen();
    }
    else {
        appcmdline("pattern not found");
        beep();
        delay(500);
        appcmdline("");
    }
}

void    cpfind()
{
    pchar st;

    st = askforstring("Enter search string [Opts: /i,/w,/r]");

    if (abortcmd) return;

    if (strlen(st) != 0) filterstring(st);      /* redefine the searchstr */
    replast = FALSE;

    appcmdline("");
    find();
}

static  void    do_replace()
{
    pLineDesc   cl = curwin->curline;
    int         lsea  = strlen(searchstr);
    int         lrep  = strlen(replacestr);
    int         last  = lastchar(cl);
    int         col   = curwin->colno;


    if (!sizeline(cl, last+lrep-lsea)) {
        appcmdline("Out of memory to replace");
        return;
    }

    memmove(cl->ptxt+col+lrep-1, cl->ptxt+col+lsea-1, last-(col+lsea-1));

    if (lrep < lsea) memset(cl->ptxt+last-(lsea-lrep), ' ', lsea-lrep);

    memmove(cl->ptxt+col-1, replacestr, lrep);
}


void    replace()
{
    pLineDesc p, q;
    int        i;

    if ((p = scanforward(curwin->curline, &curwin->colno)) != NULL) {
        curwin->curline = p;

        q = curwin->topline;
        for (i = curwin->firstlineno; i < curwin->lastlineno; i++) {    /* compare find() */
            if (q == p) break;
            else if (q->pfwd != NULL && i < curwin->lastlineno-1) q = q->pfwd;
            else break;
        }

        if (q != p) {
            curwin->topline = p;
            realign();
            linehalfdown();
        }
        realign();
        refresh_screen();

        appcmdline("Replace ? (y,n): ");

        gotoXY(curwin->colno - curwin->leftedge + 1,
                               curwin->lineno + curwin->firstlineno);

        if (toupper(getch()) == 'Y') {
            do_replace();
            curwin->colno += strlen(replacestr);
        }
        else
            curwin->colno += strlen(searchstr);

        refresh_screen();
    }
    else {
        appcmdline("pattern not found");
        beep();
        delay(500);
    }

    appcmdline("");
}


static  void    blockreplace()
{
    int        colsaved, count;
    pLineDesc  p, linesaved;

    linesaved = curwin->curline;
    colsaved  = curwin->colno;
    curwin->curline = blockfro;
    curwin->colno   = 1;
    count = 0;

    p = scanforward(curwin->curline, &curwin->colno);
    while ( p != NULL && in_block(p) ) {
        count++;
        curwin->curline = p;
        do_replace();
        curwin->colno += strlen(replacestr);
        p = scanforward(curwin->curline, &curwin->colno);
    }

    sprintf(cmdline2, "'%s' replaced '%s' %d times", replacestr, searchstr, count);
    appcmdline(cmdline2);

    curwin->curline = linesaved;
    curwin->colno = colsaved;
    unmarkblock();
    realign();
}


void    cpreplace()
{
    pchar st;

    st = askforstring("Enter search string [Opts: /i,/w,/r]");
    if (abortcmd) return;

    if (strlen(st) != 0) filterstring(st);      /* redefine the searchstr */
    else if (strlen(searchstr) == 0) return;

    st = askforstring("Enter replace string: ");
    if (abortcmd) return;

    strcpy(replacestr, st);
    replast = TRUE;
    appcmdline("Searching...");

    if ( blockdefinedincurwin() ) blockreplace();
    else replace();
}


void    edit_q()
{
    int     ch;
    char    cmdstr[] =
" Ydelstream  delToeoln  restoreLine  Find  find&replAce                     ESC ";

    write_commands(cmdstr);

    while (!keypressed()) donothing();
    if ((ch = getctrlchar()) == NULL) {         /* if fnckey, get the */
        getinput();                             /* other and discard  */
        ch = NOTAVAILABLE;
    }
    appcmdline("");

    switch(ch) {
        case ctrla: cpreplace();        break;

        case ctrlf: cpfind();           break;

        case ctrll: restoreline();      break;

        case ctrlt: deletetextright();  break;

        case ctrly: cpwindowdelete();   break;

        default:    break;
    }
}