/*
 *       ͻ
 *              Chi Programmer's Editor, Borland C++ 3.1       
 *                     
 *           Copyright (C) 1990,1991,1993                      
 *           Author: Fook H. Eng, 201 West Hilliard Avenue     
 *                   Monterey Park, CA 91754, 818 572-9220     
 *                                                             
 *       ͼ
 */
#include    <stdio.h>
#include    <stdlib.h>
#include    <process.h>
#include    <dos.h>
#include    <ctype.h>
#include    <errno.h>
#include    <string.h>
#include    "vartypes.h"
#include    "cmd.h"
#include    "qcmd.h"
#include    "utils.h"
#include    "blocks.h"
#include    "cache.h"
#include    "windows.h"
#include    "memory.h"
#include    "keys.h"
#include    "cursors.h"
#include    "screen.h"
#include    "files.h"
#include    "cursors.h"
#include    "taskinp.h"

static void execdos() {

    const  envsize = 12;
    pchar  envp[envsize];
    pchar  comspec;
    char    errbuf[80];
    int     i, prompt = FALSE;

    clear_scrn();
    printf("\rChi, a Programmer's Editor, (C)Copyright Fook H Eng, 1990,1992\n");

    //  the global array of char pointers, environ[], is defined in dos.h

    for (i = 0; i < envsize && environ[i] != NULL; i++)
        if (strstr(environ[i], "PROMPT") != NULL) {
            envp[i] = "PROMPT=Type 'EXIT' to return to editor$_$t $P$_$g";
            prompt = TRUE;
        }
        else envp[i] = environ[i];

    if (i < envsize-1 && !prompt) {
        envp[i] = "PROMPT=Type 'EXIT' to return to editor$_$t $P$_$g";
        envp[i+1] = NULL;
    }
    else if (i < envsize) envp[i] = NULL;

    set_cursor_type(TRUE);

    for (i = 0; environ[i] != NULL; i++)
        if (strstr(environ[i], "COMSPEC") != NULL) {
            comspec = environ[i] + 8;
            break;
        }

    // In earlier version, we assumed environ[0] pointed to COMSPEC.
    // This unwarranted assumption came back to haunt me in MSDOS 6.0
    // which allowed a menu of configurations on bootup, introducing
    // CONFIG as the new environ[0]

    if (spawnlpe(P_WAIT, comspec, comspec, NULL, envp) == -1) {
        strcpy(errbuf, strerror(errno));
        if (errbuf[strlen(errbuf)-1] == '\x0a')  // strerror() returns a string
            errbuf[strlen(errbuf)-1] = '\0';     // followed by an annoying
        appcmdline (errbuf);                     // linefeed. We delete it.
    }
    else clear_scrln(1);

    set_cursor_type(curwin->insertflag);
    refresh_screen();
}

static int  scroll_lock_on() { return ((peekb (0x40, 0x17) & 0x10) != NULL); }


int  key_shifted() { return ((peekb (0x40, 0x17) & 0x03) != NULL); }


static void  cpload_macros() {

    pchar   macname;

    if ((macname = askforstring("Enter macro file name")) != NULL) {
        if (load_macros(macname)) {
            sprintf(cmdline2, "macro file %s loaded", macname);
            appcmdline(cmdline2);
        }
    }
}

static void  process_arrow(int ch) {

    switch (ch) {

        case UP:    if (scroll_lock_on()) scroll_up();
                    else if (key_shifted()) upcaseline();
                    else upline();
                    break;

        case DOWN:  if (scroll_lock_on()) scroll_down();
                    else if (key_shifted()) locaseline();
                    else downline();
                    break;

        case LEFT:  if (scroll_lock_on()) scroll_left(1);
                    else if (key_shifted()) {
                        changecase();
                        leftchar();
                    }
                    else leftchar();
                    break;

        case RIGHT: if (scroll_lock_on()) scroll_right(1);
                    else if (key_shifted()) {
                        changecase();
                        rightchar();
                    }
                    else rightchar();
                    break;
    }
}

static void process_fnckey(int ch) {

    pchar pch;

    if ( in_cache(curwin->curline) < MAXCACHEBLK)  switch (ch) {

        case BACKTAB:  case ctrl_TAB:  case alt_INS:
        case DEL:      case ctrl_DEL:  case HOME:  case END:
        case F5:  case F6:  case F7:  case F8:
        case F9:  case F10: case F11: case F12:
                    beep();     return;

        default:    break;
    }

    if ( in_block(curwin->curline) )  switch (ch) {

        case F5:  case F6:  case F7:  case F8:
        case F9:  case F10: case F11: case F12:
        case ctrl_TAB:   case alt_INS:
        case ctrl_INS:   case altk:
        case HOME:       case END:
                    beep();     return;

        case LEFT:  case RIGHT:   do_blockshift(ch);    return;
        case ctrl_DEL: case DEL:  delete_block();       return;

        default:    break;
    }

    switch (ch) {
        case ctrl_INS:  insert_delline();   break;

        case altk:      insert_delblock();  break;

        case ctrl_LEFT: if (scroll_lock_on())
                            scroll_left(defnocols / 2);
                        else
                            leftword();
                        break;

        case ctrl_RIGHT:if (scroll_lock_on())
                            scroll_right(defnocols / 2);
                        else
                            rightword();
                        break;

        case ctrl_UP:   upword();   break;
        case ctrl_DOWN: downword(); break;

        case ctrl_TAB:  deletetextright();  break;
        case ctrl_DEL:  delete_line();      break;

        case alt_INS:   duplicate_line();   break;

        case ctrl_PGUP: currenttop();       break;
        case ctrl_PGDN: currentbottom();    break;
        case ctrl_HOME: windowtopfile();    break;
        case ctrl_END:  windowbottomfile(); break;

        case HOME:  homeline();     break;
        case END:   endline();      break;

        case INS:   curwin->insertflag = !curwin->insertflag;
                    set_cursor_type(curwin->insertflag);
                    break;

        case DEL:   delete_char();  break;

        case PGUP:  uppage();       break;
        case PGDN:  downpage();     break;

        case alt_PGUP:  linehalfup();       break;
        case alt_PGDN:  linehalfdown();     break;
        case alt_HOME:
        case alt_UP:    cursorhalfup();     break;
        case alt_END:
        case alt_DOWN:  cursorhalfdown();   break;
        case alt_LEFT:  cursorhalfleft();   break;
        case alt_RIGHT: cursorhalfright();  break;

        case alt_F9:    filesave(curwin->filename);
                        appcmdline("saved!");
                        windowdeletetext();     break;

        case F1:  case F2:
        case F3:  case F4:  towindowXYZ(ch);    break;

        case F5: case F6: case F7:
        case F8: case F9: case F10:
            pch = macstr[ch-F5];
            while (*pch != '\0') pokechr(*pch++);
            break;
        case F11: case F12:
            pch = macstr[ch-F11+6];
            while (*pch != '\0') pokechr(*pch++);
            break;

        case UP:   case DOWN:
        case LEFT: case RIGHT: process_arrow(ch); break;

        case alta: AsciiScreen();   break;
        case altc: C_precedence();  break;
        case altg: gotolinenum();   break;
        case alth: HelpScreen();    break;
        case altl: cpload_macros(); break;
        case alts: switch_window(); break;
        case altx: rundown = TRUE;  break;
        case altv: do_logo();       break;
        case altz: execdos();       break;
        case BACKTAB: backtab();    break;
        case altb: sprintf(cmdline2, "status line is %d, last line is %d,"
                        " window line is %d, file line is %d",
                        curwin->firstlineno, curwin->lastlineno,
                        curwin->lineno, curwin->clineno);
                   appcmdline(cmdline2);
                   break;

        default:   sprintf(cmdline2, "Non-Fn function key 0x%x", ch);
                   appcmdline(cmdline2);
                   break;
    }
    realign();
    updcurflag = TRUE;
    refresh_screen();
}

static void process_text(int ch);

static void process_command(int ch) {

    int     i, chr;

    if  ( (i = in_cache(curwin->curline)) < MAXCACHEBLK )  switch (ch) {
        case ctrlg: case ctrlh: case ctrli: case ctrlm:
        case ctrlp: case ctrlt: case ctrly: case ctrl_BS:
                    beep();     return;

        case ctrlo: uncache_text(i);
                    realign();
                    updcurflag = TRUE;
                    appcmdline("");
                    refresh_screen();
                    return;

        default:    break;
    }

    if  ( in_block(curwin->curline) )  switch (ch) {
        case ctrlg: case ctrlh: case ctrli: case ctrlm:
        case ctrlt: case ctrly: case ctrl_BS:
                    beep();     return;

        case ctrlp: cache_text();
                    realign();
                    updcurflag = TRUE;
                    refresh_screen();
                    return;

        default:    break;
    }

    switch (ch) {
        case ctrla: leftword();     break;
        case ctrlb: sprintf(cmdline2, "firstlineno %d, lastlineno %d, lineno %d",
                        curwin->firstlineno, curwin->lastlineno, curwin->lineno);
                    appcmdline(cmdline2);
                    break;
        case ctrlc: downpage();     break;
        case ctrld: rightchar();    break;
        case ctrle: upline();       break;
        case ctrlf: rightword();    break;
        case ctrlg: delete_char();  break;
        case ctrlh: delete_left_char();     break;
        case ctrli: puttab();       break;

        case ctrlk: edit_k();       break;

        case ctrll: if (replast) replace();
                    else find();    break;

        case ctrlm: newline();      break;

        case ctrln: insert_line(key_shifted());
                    break;

        case ctrlo: edit_o();       break;

        case ctrlp: appcmdline("Entering control character..");
                    if ((chr = getctrlchar()) != NOTAVAILABLE)
                        process_text(chr);
                    break;

        case ctrlq: edit_q();               break;
        case ctrlr: uppage();               break;
        case ctrls: leftchar();             break;
        case ctrlt:   delete_right_word();  break;
        case ctrl_BS: delete_left_word();   break;

        case ctrlw: scroll_up();            break;
        case ctrlx: downline();             break;
        case ctrly: delete_line();          break;
        case ctrlz: scroll_down();          break;
        case ESC:   break;

        default  :  sprintf(cmdline2, "ctrl-%c undefined", ch | 0x40);
                    appcmdline(cmdline2);
                    beep();                 break;
    }
    realign();
    updcurflag = TRUE;
    refresh_screen();
}

static REGS regs;
static int  row;
static int  col;

static void process_text(int ch) {

    int     i, cndx, bufsize;
    char    txtline[MAXLINELENGTH];
    pLineDesc  cl = curwin->curline;

    if ( (cndx = in_cache(cl)) < MAXCACHEBLK )
        switch (ch) {
            case 'o': i = askfornum("Uncache how many lines from bottom?");
                      partial_uncache(cndx, -i);
                      appcmdline("");
                      return;

            case 'O': i = askfornum("Uncache how many lines from top?");
                      partial_uncache(cndx, i);
                      appcmdline("");
                      return;

            default:  beep();   return;
        }

    if ( in_block(cl) ) { beep();  return; }

    if (curwin->colno < curwin->leftedge)
        curwin->leftedge = curwin->colno;
    else if (curwin->colno > curwin->leftedge + logscrcols - 2)
        hscroll();

    if (curwin->insertflag) {
        if (curwin->colno > cl->bufflen)
            bufsize = curwin->colno;
        else if (cl->ptxt[cl->bufflen-1] == ' ')
            bufsize = cl->bufflen;
        else
            bufsize = cl->bufflen + 1;

        if (!sizeline(cl, bufsize)) {
            appcmdline("no more memory available");
            return;
        }

        for (i = cl->bufflen-1; i >= curwin->colno; i--)
            cl->ptxt[i] = cl->ptxt[i-1];
    }
    else if (curwin->colno <= cl->bufflen)
        donothing();
    else if (!sizeline(cl, curwin->colno)) {
        appcmdline("no more memory available");
        return;
    }

    cl->ptxt[curwin->colno - 1] = ch;
    bufsize = min(logscrcols, cl->bufflen - curwin->leftedge + 1);
    memset(txtline, ' ', logscrcols);
    memmove(txtline, &cl->ptxt[curwin->leftedge - 1], bufsize);
    row = curwin->firstlineno + curwin->lineno;
    update_row(txtline, row, txtcolor);
    curwin->colno++;
    gotoXY(curwin->colno - curwin->leftedge + 1, row);

    updcurflag = TRUE;
    refresh_screen();
}

static int  keydown_at_margin() {

    mymouse.MousePressInfo(Left_Button);
    regs = mymouse.MouseResults();
    row = regs.x.dx >> 3;
    col = regs.x.cx >> 3;

    return (regs.x.ax & 0x01) && (col == 0 || col == defnocols-1 ||
                row == curwin->firstlineno || row == curwin->lastlineno-1);
}

static void keydown_task() {

    if (row == curwin->firstlineno)
        upline();
    else if (row == curwin->lastlineno-1)
        downline();
    else if (col == defnocols - 1)
        rightchar();
    else if (col == 0)
        leftchar();
}

void    process_mouse() {

    regs = mymouse.MouseResults();
    row  = (regs.x.dx >> 3) + 1;
    col  = (regs.x.cx >> 3) + 1;

    if (row <= curwin->firstlineno || row > curwin->lastlineno) {
        switch_window();
        if (row == curwin->firstlineno)
            curwin->lineno = 1;
        else if (row == curwin->lastlineno + 1)
            curwin->lineno = curwin->lastlineno - curwin->firstlineno;
        else
            curwin->lineno = row - curwin->firstlineno;
    }
    else
        curwin->lineno = row - curwin->firstlineno;

    curwin->colno  = col + curwin->leftedge - 1;

    curwin->curline = curwin->topline;
    for (int i =1; i < curwin->lineno && curwin->curline->pfwd != NULL; i++)
        curwin->curline = curwin->curline->pfwd;

    if (curwin->lineno != i) curwin->lineno = i;
    if (extra.ptr != curwin->curline) updextraline();

    delay(400);               // give him a chance to let go of mouse button
    if (keydown_at_margin()) do {
        keydown_task();
        updcurflag = TRUE;
        refresh_screen();
        delay(25);           // if not, scrolling would be too fast
    } while (keydown_at_margin());

    appcmdline("");
    updcurflag = TRUE;
    refresh_screen();
}

void    process_keyboard() {

    int     ch;

    appcmdline("");
    abortcmd = FALSE;

    if ((ch = getinput()) == NOTAVAILABLE) donothing();
    else if (ch == NULL) process_fnckey(getinput());
    else if (ch >= ' ' && ch != ctrl_BS) process_text(ch);
    else process_command(ch);

    if (extra.ptr != curwin->curline) updextraline();

    updwinsl(curwin);
    updcurflag = TRUE;
    refresh_screen();
}